@@ -18,16 +18,20 @@ from __future__ import print_function
 import logging
 
 from errno import EEXIST
-from itertools import islice, izip
+from itertools import islice
 from operator import itemgetter
 from os import mkdir
 from os.path import basename, abspath, dirname, exists, join as pathjoin
 from sys import argv as sys_argv, exit, stderr, stdout
 from textwrap import wrap
 from time import time
+from datetime import timedelta
 import optparse
 import math
 
+from six.moves import zip as izip
+from six.moves import input
+
 from swift.common import exceptions
 from swift.common.ring import RingBuilder, Ring
 from swift.common.ring.builder import MAX_BALANCE
@@ -188,8 +192,8 @@ def _set_weight_values(devs, weight):
         print('Matched more than one device:')
         for dev in devs:
             print('    %s' % format_device(dev))
-        if raw_input('Are you sure you want to update the weight for '
-                     'these %s devices? (y/N) ' % len(devs)) != 'y':
+        if input('Are you sure you want to update the weight for '
+                 'these %s devices? (y/N) ' % len(devs)) != 'y':
             print('Aborting device modifications')
             exit(EXIT_ERROR)
 
@@ -245,8 +249,8 @@ def _set_info_values(devs, change):
         print('Matched more than one device:')
         for dev in devs:
             print('    %s' % format_device(dev))
-        if raw_input('Are you sure you want to update the info for '
-                     'these %s devices? (y/N) ' % len(devs)) != 'y':
+        if input('Are you sure you want to update the info for '
+                 'these %s devices? (y/N) ' % len(devs)) != 'y':
             print('Aborting device modifications')
             exit(EXIT_ERROR)
 
@@ -290,14 +294,14 @@ def _parse_set_info_values(argvish):
             devs = builder.search_devs(parse_search_value(search_value))
             change = {}
             ip = ''
-            if len(change_value) and change_value[0].isdigit():
+            if change_value and change_value[0].isdigit():
                 i = 1
                 while (i < len(change_value) and
                        change_value[i] in '0123456789.'):
                     i += 1
                 ip = change_value[:i]
                 change_value = change_value[i:]
-            elif len(change_value) and change_value[0] == '[':
+            elif change_value and change_value.startswith('['):
                 i = 1
                 while i < len(change_value) and change_value[i] != ']':
                     i += 1
@@ -315,14 +319,14 @@ def _parse_set_info_values(argvish):
             if change_value.startswith('R'):
                 change_value = change_value[1:]
                 replication_ip = ''
-                if len(change_value) and change_value[0].isdigit():
+                if change_value and change_value[0].isdigit():
                     i = 1
                     while (i < len(change_value) and
                            change_value[i] in '0123456789.'):
                         i += 1
                     replication_ip = change_value[:i]
                     change_value = change_value[i:]
-                elif len(change_value) and change_value[0] == '[':
+                elif change_value and change_value.startswith('['):
                     i = 1
                     while i < len(change_value) and change_value[i] != ']':
                         i += 1
@@ -418,6 +422,8 @@ swift-ring-builder <builder_file> create <part_power> <replicas>
         """
 swift-ring-builder <builder_file>
     Shows information about the ring and the devices within.
+    Flags:
+        DEL - marked for removal and will be removed next rebalance.
         """
         print('%s, build version %d' % (builder_file, builder.version))
         regions = 0
@@ -439,32 +445,25 @@ swift-ring-builder <builder_file>
                   builder.parts, builder.replicas, regions, zones, dev_count,
                   balance, dispersion_trailer))
         print('The minimum number of hours before a partition can be '
-              'reassigned is %s' % builder.min_part_hours)
+              'reassigned is %s (%s remaining)' % (
+                  builder.min_part_hours,
+                  timedelta(seconds=builder.min_part_seconds_left)))
         print('The overload factor is %0.2f%% (%.6f)' % (
             builder.overload * 100, builder.overload))
         if builder.devs:
+            balance_per_dev = builder._build_balance_per_dev()
             print('Devices:    id  region  zone      ip address  port  '
                   'replication ip  replication port      name '
-                  'weight partitions balance meta')
-            weighted_parts = builder.parts * builder.replicas / \
-                sum(d['weight'] for d in builder.devs if d is not None)
-            for dev in builder.devs:
-                if dev is None:
-                    continue
-                if not dev['weight']:
-                    if dev['parts']:
-                        balance = MAX_BALANCE
-                    else:
-                        balance = 0
-                else:
-                    balance = 100.0 * dev['parts'] / \
-                        (dev['weight'] * weighted_parts) - 100.0
+                  'weight partitions balance flags meta')
+            for dev in builder._iter_devs():
+                flags = 'DEL' if dev in builder._remove_devs else ''
                 print('         %5d %7d %5d %15s %5d %15s %17d %9s %6.02f '
-                      '%10s %7.02f %s' %
+                      '%10s %7.02f %5s %s' %
                       (dev['id'], dev['region'], dev['zone'], dev['ip'],
                        dev['port'], dev['replication_ip'],
                        dev['replication_port'], dev['device'], dev['weight'],
-                       dev['parts'], balance, dev['meta']))
+                       dev['parts'], balance_per_dev[dev['id']], flags,
+                       dev['meta']))
         exit(EXIT_SUCCESS)
 
     def search():
@@ -732,8 +731,8 @@ swift-ring-builder <builder_file> search
             print('Matched more than one device:')
             for dev in devs:
                 print('    %s' % format_device(dev))
-            if raw_input('Are you sure you want to remove these %s '
-                         'devices? (y/N) ' % len(devs)) != 'y':
+            if input('Are you sure you want to remove these %s '
+                     'devices? (y/N) ' % len(devs)) != 'y':
                 print('Aborting device removals')
                 exit(EXIT_ERROR)
 
@@ -784,16 +783,25 @@ swift-ring-builder <builder_file> rebalance [options]
 
         if options.debug:
             logger = logging.getLogger("swift.ring.builder")
+            logger.disabled = False
             logger.setLevel(logging.DEBUG)
             handler = logging.StreamHandler(stdout)
             formatter = logging.Formatter("%(levelname)s: %(message)s")
             handler.setFormatter(formatter)
             logger.addHandler(handler)
 
+        if builder.min_part_seconds_left > 0 and not options.force:
+            print('No partitions could be reassigned.')
+            print('The time between rebalances must be at least '
+                  'min_part_hours: %s hours (%s remaining)' % (
+                      builder.min_part_hours,
+                      timedelta(seconds=builder.min_part_seconds_left)))
+            exit(EXIT_WARNING)
+
         devs_changed = builder.devs_changed
         try:
             last_balance = builder.get_balance()
-            parts, balance = builder.rebalance(seed=get_seed(3))
+            parts, balance, removed_devs = builder.rebalance(seed=get_seed(3))
         except exceptions.RingBuilderError as e:
             print('-' * 79)
             print("An error has occurred during ring validation. Common\n"
@@ -803,10 +811,9 @@ swift-ring-builder <builder_file> rebalance [options]
                   (e,))
             print('-' * 79)
             exit(EXIT_ERROR)
-        if not (parts or options.force):
+        if not (parts or options.force or removed_devs):
             print('No partitions could be reassigned.')
-            print('Either none need to be or none can be due to '
-                  'min_part_hours [%s].' % builder.min_part_hours)
+            print('There is no need to do so at this time')
             exit(EXIT_WARNING)
         # If we set device's weight to zero, currently balance will be set
         # special value(MAX_BALANCE) until zero weighted device return all
@@ -871,7 +878,15 @@ swift-ring-builder <builder_file> dispersion <search_filter> [options]
     --verbose option will display dispersion graph broken down by tier
 
     You can filter which tiers are evaluated to drill down using a regex
-    in the optional search_filter arguemnt.
+    in the optional search_filter arguemnt.  i.e.
+
+        swift-ring-builder <builder_file> dispersion "r\d+z\d+$" -v
+
+    ... would only display rows for the zone tiers
+
+        swift-ring-builder <builder_file> dispersion ".*\-[^/]*$" -v
+
+    ... would only display rows for the server tiers
 
     The reports columns are:
 
@@ -909,6 +924,8 @@ swift-ring-builder <builder_file> dispersion <search_filter> [options]
                                    verbose=options.verbose)
         print('Dispersion is %.06f, Balance is %.06f, Overload is %0.2f%%' % (
             builder.dispersion, builder.get_balance(), builder.overload * 100))
+        print('Required overload is %.6f%%' % (
+            builder.get_required_overload() * 100))
         if report['worst_tier']:
             status = EXIT_WARNING
             print('Worst tier is %.06f (%s)' % (report['max_dispersion'],
@@ -995,6 +1012,8 @@ swift-ring-builder <ring_file> write_builder [min_part_hours]
             min_part_hours = 24
         ring = Ring(ring_file)
         for dev in ring.devs:
+            if dev is None:
+                continue
             dev.update({
                 'parts': 0,
                 'parts_wanted': 0,
@@ -1017,10 +1036,22 @@ swift-ring-builder <ring_file> write_builder [min_part_hours]
         for parts in builder._replica2part2dev:
             for dev_id in parts:
                 builder.devs[dev_id]['parts'] += 1
-        builder._set_parts_wanted()
         builder.save(builder_file)
 
     def pretend_min_part_hours_passed():
+        """
+swift-ring-builder <builder_file> pretend_min_part_hours_passed
+    Resets the clock on the last time a rebalance happened, thus
+    circumventing the min_part_hours check.
+
+    *****************************
+    USE THIS WITH EXTREME CAUTION
+    *****************************
+
+    If you run this command and deploy rebalanced rings before a replication
+    pass completes, you may introduce unavailability in your cluster. This
+    has an end-user impact.
+        """
         builder.pretend_min_part_hours_passed()
         builder.save(builder_file)
         exit(EXIT_SUCCESS)
@@ -1130,7 +1161,7 @@ def main(arguments=None):
         print(Commands.default.__doc__.strip())
         print()
         cmds = [c for c, f in Commands.__dict__.items()
-                if f.__doc__ and c[0] != '_' and c != 'default']
+                if f.__doc__ and not c.startswith('_') and c != 'default']
         cmds.sort()
         for cmd in cmds:
             print(Commands.__dict__[cmd].__doc__.strip())
@@ -1178,12 +1209,12 @@ def main(arguments=None):
     if argv[0].endswith('-safe'):
         try:
             with lock_parent_directory(abspath(builder_file), 15):
-                Commands.__dict__.get(command, Commands.unknown.im_func)()
+                Commands.__dict__.get(command, Commands.unknown.__func__)()
         except exceptions.LockTimeout:
             print("Ring/builder dir currently locked.")
             exit(2)
     else:
-        Commands.__dict__.get(command, Commands.unknown.im_func)()
+        Commands.__dict__.get(command, Commands.unknown.__func__)()
 
 
 if __name__ == '__main__':
