@@ -17,13 +17,15 @@
 
 from contextlib import contextmanager, closing
 import hashlib
+import json
 import logging
 import os
 from uuid import uuid4
 import sys
 import time
 import errno
-import cPickle as pickle
+import six
+import six.moves.cPickle as pickle
 from swift import gettext_ as _
 from tempfile import mkstemp
 
@@ -31,7 +33,7 @@ from eventlet import sleep, Timeout
 import sqlite3
 
 from swift.common.constraints import MAX_META_COUNT, MAX_META_OVERALL_SIZE
-from swift.common.utils import json, Timestamp, renamer, \
+from swift.common.utils import Timestamp, renamer, \
     mkdirs, lock_parent_directory, fallocate
 from swift.common.exceptions import LockTimeout
 from swift.common.swob import HTTPBadRequest
@@ -48,11 +50,12 @@ PENDING_CAP = 131072
 
 
 def utf8encode(*args):
-    return [(s.encode('utf8') if isinstance(s, unicode) else s) for s in args]
+    return [(s.encode('utf8') if isinstance(s, six.text_type) else s)
+            for s in args]
 
 
 def utf8encodekeys(metadata):
-    uni_keys = [k for k in metadata if isinstance(k, unicode)]
+    uni_keys = [k for k in metadata if isinstance(k, six.text_type)]
     for k in uni_keys:
         sv = metadata[k]
         del metadata[k]
@@ -328,8 +331,10 @@ class DatabaseBroker(object):
             exc_hint = 'malformed'
         elif 'file is encrypted or is not a database' in str(exc_value):
             exc_hint = 'corrupted'
+        elif 'disk I/O error' in str(exc_value):
+            exc_hint = 'disk error while accessing'
         else:
-            raise exc_type, exc_value, exc_traceback
+            six.reraise(exc_type, exc_value, exc_traceback)
         prefix_path = os.path.dirname(self.db_dir)
         partition_path = os.path.dirname(prefix_path)
         dbs_path = os.path.dirname(partition_path)
@@ -338,12 +343,12 @@ class DatabaseBroker(object):
                                  self.db_type + 's',
                                  os.path.basename(self.db_dir))
         try:
-            renamer(self.db_dir, quar_path)
+            renamer(self.db_dir, quar_path, fsync=False)
         except OSError as e:
             if e.errno not in (errno.EEXIST, errno.ENOTEMPTY):
                 raise
             quar_path = "%s-%s" % (quar_path, uuid4().hex)
-            renamer(self.db_dir, quar_path)
+            renamer(self.db_dir, quar_path, fsync=False)
         detail = _('Quarantined %s to %s due to %s database') % \
                   (self.db_dir, quar_path, exc_hint)
         self.logger.error(detail)
@@ -732,7 +737,7 @@ class DatabaseBroker(object):
         """
         meta_count = 0
         meta_size = 0
-        for key, (value, timestamp) in metadata.iteritems():
+        for key, (value, timestamp) in metadata.items():
             key = key.lower()
             if value != '' and (key.startswith('x-account-meta') or
                                 key.startswith('x-container-meta')):
@@ -760,7 +765,7 @@ class DatabaseBroker(object):
         """
         old_metadata = self.metadata
         if set(metadata_updates).issubset(set(old_metadata)):
-            for key, (value, timestamp) in metadata_updates.iteritems():
+            for key, (value, timestamp) in metadata_updates.items():
                 if timestamp > old_metadata[key][1]:
                     break
             else:
@@ -778,7 +783,7 @@ class DatabaseBroker(object):
                     ALTER TABLE %s_stat
                     ADD COLUMN metadata TEXT DEFAULT '' """ % self.db_type)
                 md = {}
-            for key, value_timestamp in metadata_updates.iteritems():
+            for key, value_timestamp in metadata_updates.items():
                 value, timestamp = value_timestamp
                 if key not in md or timestamp > md[key][1]:
                     md[key] = value_timestamp
@@ -842,7 +847,7 @@ class DatabaseBroker(object):
             if md:
                 md = json.loads(md)
                 keys_to_delete = []
-                for key, (value, value_timestamp) in md.iteritems():
+                for key, (value, value_timestamp) in md.items():
                     if value == '' and value_timestamp < timestamp:
                         keys_to_delete.append(key)
                 if keys_to_delete:
