@@ -18,12 +18,14 @@ Internal client library for making calls directly to the servers rather than
 through the proxy.
 """
 
+import json
 import os
 import socket
-from httplib import HTTPException
 from time import time
 
 from eventlet import sleep, Timeout
+import six
+from six.moves.http_client import HTTPException
 
 from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ClientException
@@ -33,11 +35,6 @@ from swift.common.http import HTTP_NO_CONTENT, HTTP_INSUFFICIENT_STORAGE, \
 from swift.common.swob import HeaderKeyDict
 from swift.common.utils import quote
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
-
 
 class DirectClientException(ClientException):
 
@@ -53,7 +50,7 @@ class DirectClientException(ClientException):
 
 
 def _get_direct_account_container(path, stype, node, part,
-                                  account, marker=None, limit=None,
+                                  marker=None, limit=None,
                                   prefix=None, delimiter=None, conn_timeout=5,
                                   response_timeout=15):
     """Base class for get direct account and container.
@@ -108,7 +105,7 @@ def direct_get_account(node, part, account, marker=None, limit=None,
     :param marker: marker query
     :param limit: query limit
     :param prefix: prefix query
-    :param delimeter: delimeter for the query
+    :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
     :returns: a tuple of (response headers, a list of containers) The response
@@ -116,11 +113,11 @@ def direct_get_account(node, part, account, marker=None, limit=None,
     """
     path = '/' + account
     return _get_direct_account_container(path, "Account", node, part,
-                                         account, marker=None,
-                                         limit=None, prefix=None,
-                                         delimiter=None,
-                                         conn_timeout=5,
-                                         response_timeout=15)
+                                         marker=marker,
+                                         limit=limit, prefix=prefix,
+                                         delimiter=delimiter,
+                                         conn_timeout=conn_timeout,
+                                         response_timeout=response_timeout)
 
 
 def direct_delete_account(node, part, account, conn_timeout=5,
@@ -153,6 +150,7 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
     :returns: a dict containing the response's headers in a HeaderKeyDict
+    :raises ClientException: HTTP HEAD request failed
     """
     path = '/%s/%s' % (account, container)
     with Timeout(conn_timeout):
@@ -183,7 +181,7 @@ def direct_get_container(node, part, account, container, marker=None,
     :param marker: marker query
     :param limit: query limit
     :param prefix: prefix query
-    :param delimeter: delimeter for the query
+    :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
     :returns: a tuple of (response headers, a list of objects) The response
@@ -191,23 +189,36 @@ def direct_get_container(node, part, account, container, marker=None,
     """
     path = '/%s/%s' % (account, container)
     return _get_direct_account_container(path, "Container", node,
-                                         part, account, marker=None,
-                                         limit=None, prefix=None,
-                                         delimiter=None,
-                                         conn_timeout=5,
-                                         response_timeout=15)
+                                         part, marker=marker,
+                                         limit=limit, prefix=prefix,
+                                         delimiter=delimiter,
+                                         conn_timeout=conn_timeout,
+                                         response_timeout=response_timeout)
 
 
 def direct_delete_container(node, part, account, container, conn_timeout=5,
                             response_timeout=15, headers=None):
+    """
+    Delete container directly from the container server.
+
+    :param node: node dictionary from the ring
+    :param part: partition the container is on
+    :param account: account name
+    :param container: container name
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: dict to be passed into HTTPConnection headers
+    :raises ClientException: HTTP DELETE request failed
+    """
     if headers is None:
         headers = {}
 
     path = '/%s/%s' % (account, container)
+    add_timestamp = 'x-timestamp' not in (k.lower() for k in headers)
     with Timeout(conn_timeout):
         conn = http_connect(node['ip'], node['port'], node['device'], part,
                             'DELETE', path,
-                            headers=gen_headers(headers, True))
+                            headers=gen_headers(headers, add_timestamp))
     with Timeout(response_timeout):
         resp = conn.getresponse()
         resp.read()
@@ -274,6 +285,7 @@ def direct_head_object(node, part, account, container, obj, conn_timeout=5,
     :param response_timeout: timeout in seconds for getting the response
     :param headers: dict to be passed into HTTPConnection headers
     :returns: a dict containing the response's headers in a HeaderKeyDict
+    :raises ClientException: HTTP HEAD request failed
     """
     if headers is None:
         headers = {}
@@ -312,6 +324,7 @@ def direct_get_object(node, part, account, container, obj, conn_timeout=5,
     :param headers: dict to be passed into HTTPConnection headers
     :returns: a tuple of (response headers, the object's contents) The response
               headers will be a HeaderKeyDict.
+    :raises ClientException: HTTP GET request failed
     """
     if headers is None:
         headers = {}
@@ -363,6 +376,7 @@ def direct_put_object(node, part, account, container, name, contents,
     :param response_timeout: timeout in seconds for getting the response
     :param chunk_size: if defined, chunk size of data to send.
     :returns: etag from the server response
+    :raises ClientException: HTTP PUT request failed
     """
 
     path = '/%s/%s/%s' % (account, container, name)
@@ -373,7 +387,7 @@ def direct_put_object(node, part, account, container, name, contents,
     if content_length is not None:
         headers['Content-Length'] = str(content_length)
     else:
-        for n, v in headers.iteritems():
+        for n, v in headers.items():
             if n.lower() == 'content-length':
                 content_length = int(v)
     if content_type is not None:
@@ -382,9 +396,9 @@ def direct_put_object(node, part, account, container, name, contents,
         headers['Content-Type'] = 'application/octet-stream'
     if not contents:
         headers['Content-Length'] = '0'
-    if isinstance(contents, basestring):
+    if isinstance(contents, six.string_types):
         contents = [contents]
-    #Incase the caller want to insert an object with specific age
+    # Incase the caller want to insert an object with specific age
     add_ts = 'X-Timestamp' not in headers
 
     if content_length is None:
@@ -462,7 +476,7 @@ def direct_delete_object(node, part, account, container, obj,
     :param obj: object name
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
-    :returns: response from server
+    :raises ClientException: HTTP DELETE request failed
     """
     if headers is None:
         headers = {}
@@ -493,16 +507,11 @@ def retry(func, *args, **kwargs):
     :param kwargs: keyward arguments to send to func (if retries or
                    error_log are sent, they will be deleted from kwargs
                    before sending on to func)
-    :returns: restult of func
+    :returns: result of func
+    :raises ClientException: all retries failed
     """
-    retries = 5
-    if 'retries' in kwargs:
-        retries = kwargs['retries']
-        del kwargs['retries']
-    error_log = None
-    if 'error_log' in kwargs:
-        error_log = kwargs['error_log']
-        del kwargs['error_log']
+    retries = kwargs.pop('retries', 5)
+    error_log = kwargs.pop('error_log', None)
     attempts = 0
     backoff = 1
     while attempts <= retries:
@@ -525,8 +534,8 @@ def retry(func, *args, **kwargs):
     # Shouldn't actually get down here, but just in case.
     if args and 'ip' in args[0]:
         raise ClientException('Raise too many retries',
-                              http_host=args[
-                              0]['ip'], http_port=args[0]['port'],
+                              http_host=args[0]['ip'],
+                              http_port=args[0]['port'],
                               http_device=args[0]['device'])
     else:
         raise ClientException('Raise too many retries')
