@@ -44,19 +44,20 @@ version is at:
 http://github.com/memcached/memcached/blob/1.4.2/doc/protocol.txt
 """
 
-import cPickle as pickle
+import six.moves.cPickle as pickle
+import json
 import logging
+import re
 import time
 from bisect import bisect
 from swift import gettext_ as _
 from hashlib import md5
-from distutils.version import StrictVersion
 
 from eventlet.green import socket
 from eventlet.pools import Pool
-from eventlet import Timeout, __version__ as eventlet_version
+from eventlet import Timeout
+from six.moves import range
 
-from swift.common.utils import json
 
 DEFAULT_MEMCACHED_PORT = 11211
 
@@ -101,62 +102,67 @@ class MemcachePoolTimeout(Timeout):
 
 
 class MemcacheConnPool(Pool):
-    """Connection pool for Memcache Connections"""
+    """
+    Connection pool for Memcache Connections
+
+    The *server* parameter can be a hostname, an IPv4 address, or an IPv6
+    address with an optional port. If an IPv6 address is specified it **must**
+    be enclosed in [], like *[::1]* or *[::1]:11211*. This follows the accepted
+    prescription for `IPv6 host literals`_.
+
+    Examples::
+
+        memcache.local:11211
+        127.0.0.1:11211
+        [::1]:11211
+        [::1]
+
+    .. _IPv6 host literals: https://tools.ietf.org/html/rfc3986#section-3.2.2
+    """
+    IPV6_RE = re.compile("^\[(?P<address>.*)\](:(?P<port>[0-9]+))?$")
 
     def __init__(self, server, size, connect_timeout):
         Pool.__init__(self, max_size=size)
         self.server = server
         self._connect_timeout = connect_timeout
-        self._parent_class_getter = super(MemcacheConnPool, self).get
-        try:
-            # call the patched .get() if eventlet is older than 0.9.17
-            if StrictVersion(eventlet_version) < StrictVersion('0.9.17'):
-                self._parent_class_getter = self._upstream_fixed_get
-        except ValueError:
-            # "invalid" version number or otherwise error parsing version
-            pass
+
+    def _get_addr(self):
+        port = DEFAULT_MEMCACHED_PORT
+        # IPv6 addresses must be between '[]'
+        if self.server.startswith('['):
+            match = MemcacheConnPool.IPV6_RE.match(self.server)
+            if not match:
+                raise ValueError("Invalid IPv6 address: %s" % self.server)
+            host = match.group('address')
+            port = match.group('port') or port
+        else:
+            if ':' in self.server:
+                tokens = self.server.split(':')
+                if len(tokens) > 2:
+                    raise ValueError("IPv6 addresses must be between '[]'")
+                host, port = tokens
+            else:
+                host = self.server
+        return (host, port)
 
     def create(self):
-        if ':' in self.server:
-            host, port = self.server.split(':')
-        else:
-            host = self.server
-            port = DEFAULT_MEMCACHED_PORT
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        host, port = self._get_addr()
+        addrs = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
+                                   socket.SOCK_STREAM)
+        family, socktype, proto, canonname, sockaddr = addrs[0]
+        sock = socket.socket(family, socket.SOCK_STREAM)
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         with Timeout(self._connect_timeout):
-            sock.connect((host, int(port)))
+            sock.connect(sockaddr)
         return (sock.makefile(), sock)
 
     def get(self):
-        fp, sock = self._parent_class_getter()
+        fp, sock = super(MemcacheConnPool, self).get()
         if fp is None:
             # An error happened previously, so we need a new connection
             fp, sock = self.create()
         return fp, sock
 
-    # The following method is from eventlet post 0.9.16. This version
-    # properly keeps track of pool size accounting, and therefore doesn't
-    # let the pool grow without bound. This patched version is the result
-    # of commit f5e5b2bda7b442f0262ee1084deefcc5a1cc0694 in eventlet and is
-    # documented at https://bitbucket.org/eventlet/eventlet/issue/91
-    def _upstream_fixed_get(self):
-        """Return an item from the pool, when one is available.  This may
-        cause the calling greenthread to block.
-        """
-        if self.free_items:
-            return self.free_items.popleft()
-        self.current_size += 1
-        if self.current_size <= self.max_size:
-            try:
-                created = self.create()
-            except:  # noqa
-                self.current_size -= 1
-                raise
-            return created
-        self.current_size -= 1  # did not create
-        return self.channel.get()
-
 
 class MemcacheRing(object):
     """
@@ -171,7 +177,7 @@ class MemcacheRing(object):
         self._errors = dict(((serv, []) for serv in servers))
         self._error_limited = dict(((serv, 0) for serv in servers))
         for server in sorted(servers):
-            for i in xrange(NODE_WEIGHT):
+            for i in range(NODE_WEIGHT):
                 self._ring[md5hash('%s-%s' % (server, i))] = server
         self._tries = tries if tries <= len(servers) else len(servers)
         self._sorted = sorted(self._ring)
@@ -253,7 +259,7 @@ class MemcacheRing(object):
         """Returns a server connection to the pool."""
         self._client_cache[server].put((fp, sock))
 
-    def set(self, key, value, serialize=True, timeout=0, time=0,
+    def set(self, key, value, serialize=True, time=0,
             min_compress_len=0):
         """
         Set a key/value pair in memcache
@@ -263,22 +269,14 @@ class MemcacheRing(object):
         :param serialize: if True, value is serialized with JSON before sending
                           to memcache, or with pickle if configured to use
                           pickle instead of JSON (to avoid cache poisoning)
-        :param timeout: ttl in memcache, this parameter is now deprecated. It
-                        will be removed in next release of OpenStack,
-                        use time parameter instead in the future
-        :time: equivalent to timeout, this parameter is added to keep the
-               signature compatible with python-memcached interface. This
-               implementation will take this value and sign it to the
-               parameter timeout
+        :param time: the time to live
         :min_compress_len: minimum compress length, this parameter was added
                            to keep the signature compatible with
                            python-memcached interface. This implementation
                            ignores it.
         """
         key = md5hash(key)
-        if timeout:
-            logging.warn("parameter timeout has been deprecated, use time")
-        timeout = sanitize_timeout(time or timeout)
+        timeout = sanitize_timeout(time)
         flags = 0
         if serialize and self._allow_pickle:
             value = pickle.dumps(value, PICKLE_PROTOCOL)
@@ -332,7 +330,7 @@ class MemcacheRing(object):
             except (Exception, Timeout) as e:
                 self._exception_occurred(server, e, sock=sock, fp=fp)
 
-    def incr(self, key, delta=1, time=0, timeout=0):
+    def incr(self, key, delta=1, time=0):
         """
         Increments a key which has a numeric value by delta.
         If the key can't be found, it's added as delta or 0 if delta < 0.
@@ -345,22 +343,16 @@ class MemcacheRing(object):
         :param key: key
         :param delta: amount to add to the value of key (or set as the value
                       if the key is not found) will be cast to an int
-        :param time: the time to live. This parameter deprecates parameter
-                     timeout. The addition of this parameter is to make the
-                     interface consistent with set and set_multi methods
-        :param timeout: ttl in memcache, deprecated, will be removed in future
-                        OpenStack releases
+        :param time: the time to live
         :returns: result of incrementing
         :raises MemcacheConnectionError:
         """
-        if timeout:
-            logging.warn("parameter timeout has been deprecated, use time")
         key = md5hash(key)
         command = 'incr'
         if delta < 0:
             command = 'decr'
         delta = str(abs(int(delta)))
-        timeout = sanitize_timeout(time or timeout)
+        timeout = sanitize_timeout(time)
         for (server, fp, sock) in self._get_conns(key):
             try:
                 with Timeout(self._io_timeout):
@@ -388,7 +380,7 @@ class MemcacheRing(object):
                 self._exception_occurred(server, e, sock=sock, fp=fp)
         raise MemcacheConnectionError("No Memcached connections succeeded.")
 
-    def decr(self, key, delta=1, time=0, timeout=0):
+    def decr(self, key, delta=1, time=0):
         """
         Decrements a key which has a numeric value by delta. Calls incr with
         -delta.
@@ -397,18 +389,11 @@ class MemcacheRing(object):
         :param delta: amount to subtract to the value of key (or set the
                       value to 0 if the key is not found) will be cast to
                       an int
-        :param time: the time to live. This parameter depcates parameter
-                     timeout. The addition of this parameter is to make the
-                     interface consistent with set and set_multi methods
-        :param timeout: ttl in memcache, deprecated, will be removed in future
-                        OpenStack releases
+        :param time: the time to live
         :returns: result of decrementing
         :raises MemcacheConnectionError:
         """
-        if timeout:
-            logging.warn("parameter timeout has been deprecated, use time")
-
-        return self.incr(key, delta=-delta, time=(time or timeout))
+        return self.incr(key, delta=-delta, time=time)
 
     def delete(self, key):
         """
@@ -428,36 +413,27 @@ class MemcacheRing(object):
             except (Exception, Timeout) as e:
                 self._exception_occurred(server, e, sock=sock, fp=fp)
 
-    def set_multi(self, mapping, server_key, serialize=True, timeout=0,
-                  time=0, min_compress_len=0):
+    def set_multi(self, mapping, server_key, serialize=True, time=0,
+                  min_compress_len=0):
         """
         Sets multiple key/value pairs in memcache.
 
-        :param mapping: dictonary of keys and values to be set in memcache
+        :param mapping: dictionary of keys and values to be set in memcache
         :param servery_key: key to use in determining which server in the ring
                             is used
         :param serialize: if True, value is serialized with JSON before sending
                           to memcache, or with pickle if configured to use
                           pickle instead of JSON (to avoid cache poisoning)
-        :param timeout: ttl for memcache. This parameter is now deprecated, it
-                        will be removed in next release of OpenStack, use time
-                        parameter instead in the future
-        :time: equalvent to timeout, this parameter is added to keep the
-               signature compatible with python-memcached interface. This
-               implementation will take this value and sign it to parameter
-               timeout
+        :param time: the time to live
         :min_compress_len: minimum compress length, this parameter was added
                            to keep the signature compatible with
                            python-memcached interface. This implementation
                            ignores it
         """
-        if timeout:
-            logging.warn("parameter timeout has been deprecated, use time")
-
         server_key = md5hash(server_key)
-        timeout = sanitize_timeout(time or timeout)
+        timeout = sanitize_timeout(time)
         msg = ''
-        for key, value in mapping.iteritems():
+        for key, value in mapping.items():
             key = md5hash(key)
             flags = 0
             if serialize and self._allow_pickle:
@@ -473,7 +449,7 @@ class MemcacheRing(object):
                 with Timeout(self._io_timeout):
                     sock.sendall(msg)
                     # Wait for the set to complete
-                    for _ in range(len(mapping)):
+                    for line in range(len(mapping)):
                         fp.readline()
                     self._return_conn(server, fp, sock)
                     return
