@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import errno
+import json
 import os
 import time
 from swift import gettext_ as _
@@ -21,7 +22,7 @@ from swift import gettext_ as _
 from swift import __version__ as swiftver
 from swift.common.storage_policy import POLICIES
 from swift.common.swob import Request, Response
-from swift.common.utils import get_logger, config_true_value, json, \
+from swift.common.utils import get_logger, config_true_value, \
     SWIFT_CONF_FILE
 from swift.common.constraints import check_mount
 from resource import getpagesize
