@@ -17,13 +17,14 @@ from __future__ import print_function
 
 from time import time
 from traceback import format_exc
-from urllib import unquote
 from uuid import uuid4
 from hashlib import sha1
 import hmac
 import base64
 
 from eventlet import Timeout
+import six
+from six.moves.urllib.parse import unquote
 from swift.common.swob import Response, Request
 from swift.common.swob import HTTPBadRequest, HTTPForbidden, HTTPNotFound, \
     HTTPUnauthorized
@@ -71,16 +72,16 @@ class TempAuth(object):
 
     The reseller prefix specifies which parts of the account namespace this
     middleware is responsible for managing authentication and authorization.
-    By default, the prefix is AUTH so accounts and tokens are prefixed
-    by AUTH_. When a request's token and/or path start with AUTH_, this
+    By default, the prefix is 'AUTH' so accounts and tokens are prefixed
+    by 'AUTH\_'. When a request's token and/or path start with 'AUTH\_', this
     middleware knows it is responsible.
 
     We allow the reseller prefix to be a list. In tempauth, the first item
     in the list is used as the prefix for tokens and user groups. The
     other prefixes provide alternate accounts that user's can access. For
     example if the reseller prefix list is 'AUTH, OTHER', a user with
-    admin access to AUTH_account also has admin access to
-    OTHER_account.
+    admin access to 'AUTH_account' also has admin access to
+    'OTHER_account'.
 
     Required Group:
 
@@ -98,7 +99,7 @@ class TempAuth(object):
     is not processed.
 
     The X-Service-Token is useful when combined with multiple reseller prefix
-    items. In the following configuration, accounts prefixed SERVICE_
+    items. In the following configuration, accounts prefixed 'SERVICE\_'
     are only accessible if X-Auth-Token is from the end-user and
     X-Service-Token is from the ``glance`` user::
 
@@ -176,9 +177,9 @@ class TempAuth(object):
                                 '"/auth/" (Non-empty auth prefix path '
                                 'is required)' % self.auth_prefix)
             self.auth_prefix = '/auth/'
-        if self.auth_prefix[0] != '/':
+        if not self.auth_prefix.startswith('/'):
             self.auth_prefix = '/' + self.auth_prefix
-        if self.auth_prefix[-1] != '/':
+        if not self.auth_prefix.endswith('/'):
             self.auth_prefix += '/'
         self.token_life = int(conf.get('token_life', 86400))
         self.allow_overrides = config_true_value(
@@ -428,10 +429,12 @@ class TempAuth(object):
         try:
             acls = acls_from_account_info(info)
         except ValueError as e1:
-            self.logger.warn("Invalid ACL stored in metadata: %r" % e1)
+            self.logger.warning("Invalid ACL stored in metadata: %r" % e1)
             return None
         except NotImplementedError as e2:
-            self.logger.warn("ACL version exceeds middleware version: %r" % e2)
+            self.logger.warning(
+                "ACL version exceeds middleware version: %r"
+                % e2)
             return None
         return acls
 
@@ -460,7 +463,7 @@ class TempAuth(object):
             if not isinstance(result[key], list):
                 return "Value for key '%s' must be a list" % key
             for grantee in result[key]:
-                if not isinstance(grantee, basestring):
+                if not isinstance(grantee, six.string_types):
                     return "Elements of '%s' list must be strings" % key
 
         # Everything looks fine, no errors found
@@ -628,7 +631,8 @@ class TempAuth(object):
         req.start_time = time()
         handler = None
         try:
-            version, account, user, _junk = req.split_path(1, 4, True)
+            version, account, user, _junk = split_path(req.path_info,
+                                                       1, 4, True)
         except ValueError:
             self.logger.increment('errors')
             return HTTPNotFound(request=req)
