@@ -21,7 +21,6 @@ import os
 import multiprocessing
 import time
 import traceback
-import rfc822
 import socket
 import math
 from swift import gettext_ as _
@@ -33,14 +32,15 @@ from eventlet.greenthread import spawn
 from swift.common.utils import public, get_logger, \
     config_true_value, timing_stats, replication, \
     normalize_delete_at_timestamp, get_log_line, Timestamp, \
-    get_expirer_container, iter_multipart_mime_documents
+    get_expirer_container, parse_mime_headers, \
+    iter_multipart_mime_documents
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_object_creation, \
     valid_timestamp, check_utf8
 from swift.common.exceptions import ConnectionTimeout, DiskFileQuarantined, \
     DiskFileNotExist, DiskFileCollision, DiskFileNoSpace, DiskFileDeleted, \
     DiskFileDeviceUnavailable, DiskFileExpired, ChunkReadTimeout, \
-    DiskFileXattrNotSupported
+    ChunkReadError, DiskFileXattrNotSupported
 from swift.obj import ssync_receiver
 from swift.common.http import is_success
 from swift.common.base_storage_server import BaseStorageServer
@@ -60,7 +60,7 @@ def iter_mime_headers_and_bodies(wsgi_input, mime_boundary, read_chunk_size):
         wsgi_input, mime_boundary, read_chunk_size)
 
     for file_like in mime_documents_iter:
-        hdrs = HeaderKeyDict(rfc822.Message(file_like, 0))
+        hdrs = parse_mime_headers(file_like)
         yield (hdrs, file_like)
 
 
@@ -405,8 +405,10 @@ class ObjectController(BaseStorageServer):
                 if commit_hdrs.get('X-Document', None) == "put commit":
                     rcvd_commit = True
             drain(commit_iter, self.network_chunk_size, self.client_timeout)
-        except ChunkReadTimeout:
+        except ChunkReadError:
             raise HTTPClientDisconnect()
+        except ChunkReadTimeout:
+            raise HTTPRequestTimeout()
         except StopIteration:
             raise HTTPBadRequest(body="couldn't find PUT commit MIME doc")
         return rcvd_commit
@@ -415,16 +417,20 @@ class ObjectController(BaseStorageServer):
         try:
             with ChunkReadTimeout(self.client_timeout):
                 footer_hdrs, footer_iter = next(mime_documents_iter)
-        except ChunkReadTimeout:
+        except ChunkReadError:
             raise HTTPClientDisconnect()
+        except ChunkReadTimeout:
+            raise HTTPRequestTimeout()
         except StopIteration:
             raise HTTPBadRequest(body="couldn't find footer MIME doc")
 
         timeout_reader = self._make_timeout_reader(footer_iter)
         try:
             footer_body = ''.join(iter(timeout_reader, ''))
-        except ChunkReadTimeout:
+        except ChunkReadError:
             raise HTTPClientDisconnect()
+        except ChunkReadTimeout:
+            raise HTTPRequestTimeout()
 
         footer_md5 = footer_hdrs.get('Content-MD5')
         if not footer_md5:
@@ -481,7 +487,11 @@ class ObjectController(BaseStorageServer):
         self._preserve_slo_manifest(metadata, orig_metadata)
         metadata.update(val for val in request.headers.items()
                         if is_user_meta('object', val[0]))
-        for header_key in self.allowed_headers:
+        headers_to_copy = (
+            request.headers.get(
+                'X-Backend-Replication-Headers', '').split() +
+            list(self.allowed_headers))
+        for header_key in headers_to_copy:
             if header_key in request.headers:
                 header_caps = header_key.title()
                 metadata[header_caps] = request.headers[header_key]
@@ -543,10 +553,12 @@ class ObjectController(BaseStorageServer):
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
             orig_metadata = disk_file.read_metadata()
+            orig_timestamp = disk_file.data_timestamp
         except DiskFileXattrNotSupported:
             return HTTPInsufficientStorage(drive=device, request=request)
         except (DiskFileNotExist, DiskFileQuarantined):
             orig_metadata = {}
+            orig_timestamp = Timestamp(0)
 
         # Checks for If-None-Match
         if request.if_none_match is not None and orig_metadata:
@@ -557,7 +569,6 @@ class ObjectController(BaseStorageServer):
                 # The current ETag matches, so return 412
                 return HTTPPreconditionFailed(request=request)
 
-        orig_timestamp = Timestamp(orig_metadata.get('X-Timestamp', 0))
         if orig_timestamp >= req_timestamp:
             return HTTPConflict(
                 request=request,
@@ -609,6 +620,8 @@ class ObjectController(BaseStorageServer):
                                 request.environ['wsgi.input'],
                                 mime_boundary, self.network_chunk_size)
                             _junk_hdrs, obj_input = next(mime_documents_iter)
+                    except ChunkReadError:
+                        return HTTPClientDisconnect(request=request)
                     except ChunkReadTimeout:
                         return HTTPRequestTimeout(request=request)
 
@@ -622,6 +635,8 @@ class ObjectController(BaseStorageServer):
                         etag.update(chunk)
                         upload_size = writer.write(chunk)
                         elapsed_time += time.time() - start_time
+                except ChunkReadError:
+                    return HTTPClientDisconnect(request=request)
                 except ChunkReadTimeout:
                     return HTTPRequestTimeout(request=request)
                 if upload_size:
@@ -682,8 +697,10 @@ class ObjectController(BaseStorageServer):
                             _junk_hdrs, _junk_body = next(mime_documents_iter)
                         drain(_junk_body, self.network_chunk_size,
                               self.client_timeout)
-                except ChunkReadTimeout:
+                except ChunkReadError:
                     raise HTTPClientDisconnect()
+                except ChunkReadTimeout:
+                    raise HTTPRequestTimeout()
                 except StopIteration:
                     pass
 
@@ -844,7 +861,7 @@ class ObjectController(BaseStorageServer):
             orig_metadata = {}
             response_class = HTTPNotFound
         else:
-            orig_timestamp = Timestamp(orig_metadata.get('X-Timestamp', 0))
+            orig_timestamp = disk_file.data_timestamp
             if orig_timestamp < req_timestamp:
                 response_class = HTTPNoContent
             else:
@@ -878,7 +895,10 @@ class ObjectController(BaseStorageServer):
                                   container, obj, request, device,
                                   policy)
         if orig_timestamp < req_timestamp:
-            disk_file.delete(req_timestamp)
+            try:
+                disk_file.delete(req_timestamp)
+            except DiskFileNoSpace:
+                return HTTPInsufficientStorage(drive=device, request=request)
             self.container_update(
                 'DELETE', account, container, obj, request,
                 HeaderKeyDict({'x-timestamp': req_timestamp.internal}),
