@@ -14,18 +14,18 @@
 # limitations under the License.
 
 import hashlib
+import json
 import os
 import random
 import socket
 import time
-import urllib
 
-import simplejson as json
-from nose import SkipTest
+from unittest2 import SkipTest
 from xml.dom import minidom
 
 import six
 from six.moves import http_client
+from six.moves import urllib
 from swiftclient import get_auth
 
 from swift.common import constraints
@@ -220,7 +220,7 @@ class Connection(object):
             return '/' + self.storage_url.split('/')[1]
 
         if path:
-            quote = urllib.quote
+            quote = urllib.parse.quote
             if cfg.get('no_quote') or cfg.get('no_path_quote'):
                 quote = lambda x: x
             return '%s/%s' % (self.storage_url,
@@ -260,7 +260,7 @@ class Connection(object):
             path = self.make_path(path, cfg=cfg)
         headers = self.make_headers(hdrs, cfg=cfg)
         if isinstance(parms, dict) and parms:
-            quote = urllib.quote
+            quote = urllib.parse.quote
             if cfg.get('no_quote') or cfg.get('no_parms_quote'):
                 quote = lambda x: x
             query_args = ['%s=%s' % (quote(x), quote(str(y)))
@@ -328,7 +328,7 @@ class Connection(object):
             headers.pop('Content-Length', None)
 
         if isinstance(parms, dict) and parms:
-            quote = urllib.quote
+            quote = urllib.parse.quote
             if cfg.get('no_quote') or cfg.get('no_parms_quote'):
                 quote = lambda x: x
             query_args = ['%s=%s' % (quote(x), quote(str(y)))
@@ -637,6 +637,7 @@ class File(Base):
 
         self.chunked_write_in_progress = False
         self.content_type = None
+        self.content_range = None
         self.size = None
         self.metadata = {}
 
@@ -698,7 +699,7 @@ class File(Base):
         headers.update(hdrs)
 
         if 'Destination' in headers:
-            headers['Destination'] = urllib.quote(headers['Destination'])
+            headers['Destination'] = urllib.parse.quote(headers['Destination'])
 
         return self.conn.make_request('COPY', self.path, hdrs=headers,
                                       parms=parms) == 201
@@ -722,9 +723,9 @@ class File(Base):
 
         if 'Destination-Account' in headers:
             headers['Destination-Account'] = \
-                urllib.quote(headers['Destination-Account'])
+                urllib.parse.quote(headers['Destination-Account'])
         if 'Destination' in headers:
-            headers['Destination'] = urllib.quote(headers['Destination'])
+            headers['Destination'] = urllib.parse.quote(headers['Destination'])
 
         return self.conn.make_request('COPY', self.path, hdrs=headers,
                                       parms=parms) == 201
@@ -838,6 +839,8 @@ class File(Base):
         for hdr in self.conn.response.getheaders():
             if hdr[0].lower() == 'content-type':
                 self.content_type = hdr[1]
+            if hdr[0].lower() == 'content-range':
+                self.content_range = hdr[1]
 
         if hasattr(buffer, 'write'):
             scratch = self.conn.response.read(8192)
@@ -1018,3 +1021,26 @@ class File(Base):
             raise ResponseError(self.conn.response)
         self.md5 = self.compute_md5sum(six.StringIO(data))
         return resp
+
+    def post(self, hdrs=None, parms=None, cfg=None, return_resp=False):
+        if hdrs is None:
+            hdrs = {}
+        if parms is None:
+            parms = {}
+        if cfg is None:
+            cfg = {}
+
+        headers = self.make_headers(cfg=cfg)
+        headers.update(hdrs)
+
+        self.conn.make_request('POST', self.path, hdrs=headers,
+                               parms=parms, cfg=cfg)
+
+        if self.conn.response.status not in (201, 202):
+            raise ResponseError(self.conn.response, 'POST',
+                                self.conn.make_path(self.path))
+
+        if return_resp:
+            return self.conn.response
+
+        return True
