@@ -198,10 +198,12 @@ def get_ring(ring_name, required_replicas, required_devices,
     if ring.replica_count != required_replicas:
         raise SkipTest('%s has %s replicas instead of %s' % (
             ring.serialized_path, ring.replica_count, required_replicas))
-    if len(ring.devs) != required_devices:
+
+    devs = [dev for dev in ring.devs if dev is not None]
+    if len(devs) != required_devices:
         raise SkipTest('%s has %s devices instead of %s' % (
             ring.serialized_path, len(ring.devs), required_devices))
-    for dev in ring.devs:
+    for dev in devs:
         # verify server is exposing mounted device
         ipport = (dev['ip'], dev['port'])
         _, server_number = get_server_number(ipport, ipport2server)
