@@ -12,16 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import unittest
 import uuid
-from urlparse import urlparse
 import random
 from nose import SkipTest
+import unittest
 
-from swiftclient import client
+from six.moves.urllib.parse import urlparse
+from swiftclient import client, ClientException
 
+from swift.common.http import HTTP_NOT_FOUND
 from swift.common.manager import Manager
-from test.probe.common import kill_servers, reset_environment, ENABLED_POLICIES
+from test.probe.common import ReplProbeTest, ENABLED_POLICIES
 
 
 def get_current_realm_cluster(url):
@@ -43,36 +44,33 @@ def get_current_realm_cluster(url):
     raise SkipTest('Unable find current realm cluster')
 
 
-class TestContainerSync(unittest.TestCase):
+class TestContainerSync(ReplProbeTest):
 
     def setUp(self):
-        (self.pids, self.port2server, self.account_ring, self.container_ring,
-         self.object_ring, self.policy, self.url, self.token,
-         self.account, self.configs) = reset_environment()
+        super(TestContainerSync, self).setUp()
         self.realm, self.cluster = get_current_realm_cluster(self.url)
 
-    def tearDown(self):
-        kill_servers(self.port2server, self.pids)
-
-    def test_sync(self):
-        base_headers = {'X-Container-Sync-Key': 'secret'}
-
+    def _setup_synced_containers(self, skey='secret', dkey='secret'):
         # setup dest container
         dest_container = 'dest-container-%s' % uuid.uuid4()
-        dest_headers = base_headers.copy()
+        dest_headers = {}
         dest_policy = None
         if len(ENABLED_POLICIES) > 1:
             dest_policy = random.choice(ENABLED_POLICIES)
             dest_headers['X-Storage-Policy'] = dest_policy.name
+        if dkey is not None:
+            dest_headers['X-Container-Sync-Key'] = dkey
         client.put_container(self.url, self.token, dest_container,
                              headers=dest_headers)
 
         # setup source container
         source_container = 'source-container-%s' % uuid.uuid4()
-        source_headers = base_headers.copy()
+        source_headers = {}
         sync_to = '//%s/%s/%s/%s' % (self.realm, self.cluster, self.account,
                                      dest_container)
         source_headers['X-Container-Sync-To'] = sync_to
+        if skey is not None:
+            source_headers['X-Container-Sync-Key'] = skey
         if dest_policy:
             source_policy = random.choice([p for p in ENABLED_POLICIES
                                            if p is not dest_policy])
@@ -80,6 +78,11 @@ class TestContainerSync(unittest.TestCase):
         client.put_container(self.url, self.token, source_container,
                              headers=source_headers)
 
+        return source_container, dest_container
+
+    def test_sync(self):
+        source_container, dest_container = self._setup_synced_containers()
+
         # upload to source
         object_name = 'object-%s' % uuid.uuid4()
         client.put_object(self.url, self.token, source_container, object_name,
@@ -88,12 +91,63 @@ class TestContainerSync(unittest.TestCase):
         # cycle container-sync
         Manager(['container-sync']).once()
 
-        # retrieve from sync'd container
-        headers, body = client.get_object(self.url, self.token,
-                                          dest_container, object_name)
+        _junk, body = client.get_object(self.url, self.token,
+                                        dest_container, object_name)
         self.assertEqual(body, 'test-body')
 
+    def test_sync_lazy_skey(self):
+        # Create synced containers, but with no key at source
+        source_container, dest_container =\
+            self._setup_synced_containers(None, 'secret')
+
+        # upload to source
+        object_name = 'object-%s' % uuid.uuid4()
+        client.put_object(self.url, self.token, source_container, object_name,
+                          'test-body')
+
+        # cycle container-sync, nothing should happen
+        Manager(['container-sync']).once()
+        with self.assertRaises(ClientException) as err:
+            _junk, body = client.get_object(self.url, self.token,
+                                            dest_container, object_name)
+        self.assertEqual(err.exception.http_status, HTTP_NOT_FOUND)
+
+        # amend source key
+        source_headers = {'X-Container-Sync-Key': 'secret'}
+        client.put_container(self.url, self.token, source_container,
+                             headers=source_headers)
+        # cycle container-sync, should replicate
+        Manager(['container-sync']).once()
+        _junk, body = client.get_object(self.url, self.token,
+                                        dest_container, object_name)
+        self.assertEqual(body, 'test-body')
+
+    def test_sync_lazy_dkey(self):
+        # Create synced containers, but with no key at dest
+        source_container, dest_container =\
+            self._setup_synced_containers('secret', None)
+
+        # upload to source
+        object_name = 'object-%s' % uuid.uuid4()
+        client.put_object(self.url, self.token, source_container, object_name,
+                          'test-body')
+
+        # cycle container-sync, nothing should happen
+        Manager(['container-sync']).once()
+        with self.assertRaises(ClientException) as err:
+            _junk, body = client.get_object(self.url, self.token,
+                                            dest_container, object_name)
+        self.assertEqual(err.exception.http_status, HTTP_NOT_FOUND)
+
+        # amend dest key
+        dest_headers = {'X-Container-Sync-Key': 'secret'}
+        client.put_container(self.url, self.token, dest_container,
+                             headers=dest_headers)
+        # cycle container-sync, should replicate
+        Manager(['container-sync']).once()
+        _junk, body = client.get_object(self.url, self.token,
+                                        dest_container, object_name)
+        self.assertEqual(body, 'test-body')
 
 if __name__ == "__main__":
-    get_current_realm_cluster('http://localhost:8080')
     unittest.main()
