@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (c) 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,22 +15,24 @@
 # limitations under the License.
 
 import numbers
+from six.moves import urllib
 import unittest
 import os
 import tarfile
-import urllib
 import zlib
 import mock
+import six
+from six import BytesIO
 from shutil import rmtree
 from tempfile import mkdtemp
-from StringIO import StringIO
 from eventlet import sleep
 from mock import patch, call
-from swift.common import utils
+from test.unit.common.middleware.helpers import FakeSwift
+from swift.common import utils, constraints
 from swift.common.middleware import bulk
-from swift.common.swob import Request, Response, HTTPException
+from swift.common.swob import Request, Response, HTTPException, \
+    HTTPNoContent, HTTPCreated, HeaderKeyDict
 from swift.common.http import HTTP_NOT_FOUND, HTTP_UNAUTHORIZED
-from swift.common.utils import json
 
 
 class FakeApp(object):
@@ -96,11 +99,11 @@ def build_dir_tree(start_path, tree_obj):
         for obj in tree_obj:
             build_dir_tree(start_path, obj)
     if isinstance(tree_obj, dict):
-        for dir_name, obj in tree_obj.iteritems():
+        for dir_name, obj in tree_obj.items():
             dir_path = os.path.join(start_path, dir_name)
             os.mkdir(dir_path)
             build_dir_tree(dir_path, obj)
-    if isinstance(tree_obj, unicode):
+    if isinstance(tree_obj, six.text_type):
         tree_obj = tree_obj.encode('utf8')
     if isinstance(tree_obj, str):
         obj_path = os.path.join(start_path, tree_obj)
@@ -113,13 +116,13 @@ def build_tar_tree(tar, start_path, tree_obj, base_path=''):
         for obj in tree_obj:
             build_tar_tree(tar, start_path, obj, base_path=base_path)
     if isinstance(tree_obj, dict):
-        for dir_name, obj in tree_obj.iteritems():
+        for dir_name, obj in tree_obj.items():
             dir_path = os.path.join(start_path, dir_name)
             tar_info = tarfile.TarInfo(dir_path[len(base_path):])
             tar_info.type = tarfile.DIRTYPE
             tar.addfile(tar_info)
             build_tar_tree(tar, dir_path, obj, base_path=base_path)
-    if isinstance(tree_obj, unicode):
+    if isinstance(tree_obj, six.text_type):
         tree_obj = tree_obj.encode('utf8')
     if isinstance(tree_obj, str):
         obj_path = os.path.join(start_path, tree_obj)
@@ -127,6 +130,104 @@ def build_tar_tree(tar, start_path, tree_obj, base_path=''):
         tar.addfile(tar_info)
 
 
+class TestUntarMetadata(unittest.TestCase):
+    def setUp(self):
+        self.app = FakeSwift()
+        self.bulk = bulk.filter_factory({})(self.app)
+        self.testdir = mkdtemp(suffix='tmp_test_bulk')
+
+    def tearDown(self):
+        rmtree(self.testdir, ignore_errors=1)
+
+    def test_extract_metadata(self):
+        self.app.register('HEAD', '/v1/a/c?extract-archive=tar',
+                          HTTPNoContent, {}, None)
+        self.app.register('PUT', '/v1/a/c/obj1?extract-archive=tar',
+                          HTTPCreated, {}, None)
+        self.app.register('PUT', '/v1/a/c/obj2?extract-archive=tar',
+                          HTTPCreated, {}, None)
+
+        # It's a real pain to instantiate TarInfo objects directly; they
+        # really want to come from a file on disk or a tarball. So, we write
+        # out some files and add pax headers to them as they get placed into
+        # the tarball.
+        with open(os.path.join(self.testdir, "obj1"), "w") as fh1:
+            fh1.write("obj1 contents\n")
+        with open(os.path.join(self.testdir, "obj2"), "w") as fh2:
+            fh2.write("obj2 contents\n")
+
+        tar_ball = BytesIO()
+        tar_file = tarfile.TarFile.open(fileobj=tar_ball, mode="w",
+                                        format=tarfile.PAX_FORMAT)
+
+        # With GNU tar 1.27.1 or later (possibly 1.27 as well), a file with
+        # extended attribute user.thingy = dingy gets put into the tarfile
+        # with pax_headers containing key/value pair
+        # (SCHILY.xattr.user.thingy, dingy), both unicode strings (py2: type
+        # unicode, not type str).
+        #
+        # With BSD tar (libarchive), you get key/value pair
+        # (LIBARCHIVE.xattr.user.thingy, dingy), which strikes me as
+        # gratuitous incompatibility.
+        #
+        # Still, we'll support uploads with both. Just heap more code on the
+        # problem until you can forget it's under there.
+        with open(os.path.join(self.testdir, "obj1")) as fh1:
+            tar_info1 = tar_file.gettarinfo(fileobj=fh1,
+                                            arcname="obj1")
+            tar_info1.pax_headers[u'SCHILY.xattr.user.mime_type'] = \
+                u'application/food-diary'
+            tar_info1.pax_headers[u'SCHILY.xattr.user.meta.lunch'] = \
+                u'sopa de albóndigas'
+            tar_info1.pax_headers[
+                u'SCHILY.xattr.user.meta.afternoon-snack'] = \
+                u'gigantic bucket of coffee'
+            tar_file.addfile(tar_info1, fh1)
+
+        with open(os.path.join(self.testdir, "obj2")) as fh2:
+            tar_info2 = tar_file.gettarinfo(fileobj=fh2,
+                                            arcname="obj2")
+            tar_info2.pax_headers[
+                u'LIBARCHIVE.xattr.user.meta.muppet'] = u'bert'
+            tar_info2.pax_headers[
+                u'LIBARCHIVE.xattr.user.meta.cat'] = u'fluffy'
+            tar_info2.pax_headers[
+                u'LIBARCHIVE.xattr.user.notmeta'] = u'skipped'
+            tar_file.addfile(tar_info2, fh2)
+
+        tar_ball.seek(0)
+
+        req = Request.blank('/v1/a/c?extract-archive=tar')
+        req.environ['REQUEST_METHOD'] = 'PUT'
+        req.environ['wsgi.input'] = tar_ball
+        req.headers['transfer-encoding'] = 'chunked'
+        req.headers['accept'] = 'application/json;q=1.0'
+
+        resp = req.get_response(self.bulk)
+        self.assertEqual(resp.status_int, 200)
+
+        # sanity check to make sure the upload worked
+        upload_status = utils.json.loads(resp.body)
+        self.assertEqual(upload_status['Number Files Created'], 2)
+
+        put1_headers = HeaderKeyDict(self.app.calls_with_headers[1][2])
+        self.assertEqual(
+            put1_headers.get('Content-Type'),
+            'application/food-diary')
+        self.assertEqual(
+            put1_headers.get('X-Object-Meta-Lunch'),
+            'sopa de alb\xc3\xb3ndigas')
+        self.assertEqual(
+            put1_headers.get('X-Object-Meta-Afternoon-Snack'),
+            'gigantic bucket of coffee')
+
+        put2_headers = HeaderKeyDict(self.app.calls_with_headers[2][2])
+        self.assertEqual(put2_headers.get('X-Object-Meta-Muppet'), 'bert')
+        self.assertEqual(put2_headers.get('X-Object-Meta-Cat'), 'fluffy')
+        self.assertEqual(put2_headers.get('Content-Type'), None)
+        self.assertEqual(put2_headers.get('X-Object-Meta-Blah'), None)
+
+
 class TestUntar(unittest.TestCase):
 
     def setUp(self):
@@ -147,15 +248,15 @@ class TestUntar(unittest.TestCase):
 
     def test_create_container_for_path(self):
         req = Request.blank('/')
-        self.assertEquals(
+        self.assertEqual(
             self.bulk.create_container(req, '/create_cont/acc/cont'),
             True)
-        self.assertEquals(self.app.calls, 2)
+        self.assertEqual(self.app.calls, 2)
         self.assertRaises(
             bulk.CreateContainerError,
             self.bulk.create_container,
             req, '/create_cont_fail/acc/cont')
-        self.assertEquals(self.app.calls, 3)
+        self.assertEqual(self.app.calls, 3)
 
     def test_extract_tar_works(self):
         # On systems where $TMPDIR is long (like OS X), we need to do this
@@ -187,8 +288,8 @@ class TestUntar(unittest.TestCase):
                 os.path.join(self.testdir, 'tar_works.tar' + extension))
             req.headers['transfer-encoding'] = 'chunked'
             resp_body = self.handle_extract_and_iter(req, compress_format)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Number Files Created'], 6)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Number Files Created'], 6)
 
             # test out xml
             req = Request.blank('/tar_works/acc/cont/')
@@ -197,10 +298,12 @@ class TestUntar(unittest.TestCase):
             req.headers['transfer-encoding'] = 'chunked'
             resp_body = self.handle_extract_and_iter(
                 req, compress_format, 'application/xml')
-            self.assert_('<response_status>201 Created</response_status>' in
-                         resp_body)
-            self.assert_('<number_files_created>6</number_files_created>' in
-                         resp_body)
+            self.assertTrue(
+                '<response_status>201 Created</response_status>' in
+                resp_body)
+            self.assertTrue(
+                '<number_files_created>6</number_files_created>' in
+                resp_body)
 
             # test out nonexistent format
             req = Request.blank('/tar_works/acc/cont/?extract-archive=tar',
@@ -216,7 +319,7 @@ class TestUntar(unittest.TestCase):
             app_iter = self.bulk(req.environ, fake_start_response)
             resp_body = ''.join([i for i in app_iter])
 
-            self.assert_('Response Status: 406' in resp_body)
+            self.assertTrue('Response Status: 406' in resp_body)
 
     def test_extract_call(self):
         base_name = 'base_works_gz'
@@ -239,7 +342,7 @@ class TestUntar(unittest.TestCase):
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar.gz'))
         self.bulk(req.environ, fake_start_response)
-        self.assertEquals(self.app.calls, 1)
+        self.assertEqual(self.app.calls, 1)
 
         self.app.calls = 0
         req.environ['wsgi.input'] = open(
@@ -248,7 +351,7 @@ class TestUntar(unittest.TestCase):
         req.method = 'PUT'
         app_iter = self.bulk(req.environ, fake_start_response)
         list(app_iter)  # iter over resp
-        self.assertEquals(self.app.calls, 7)
+        self.assertEqual(self.app.calls, 7)
 
         self.app.calls = 0
         req = Request.blank('/tar_works/acc/cont/?extract-archive=bad')
@@ -257,7 +360,7 @@ class TestUntar(unittest.TestCase):
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar.gz'))
         t = self.bulk(req.environ, fake_start_response)
-        self.assertEquals(t[0], "Unsupported archive format")
+        self.assertEqual(t[0], "Unsupported archive format")
 
         tar = tarfile.open(name=os.path.join(self.testdir,
                                              'tar_works.tar'),
@@ -272,7 +375,7 @@ class TestUntar(unittest.TestCase):
             os.path.join(self.testdir, 'tar_works.tar'))
         app_iter = self.bulk(req.environ, fake_start_response)
         list(app_iter)  # iter over resp
-        self.assertEquals(self.app.calls, 7)
+        self.assertEqual(self.app.calls, 7)
 
     def test_bad_container(self):
         req = Request.blank('/invalid/', body='')
@@ -320,8 +423,8 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Files Created'], 4)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Files Created'], 4)
 
     def test_extract_tar_fail_cont_401(self):
         self.build_tar()
@@ -331,10 +434,10 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        self.assertEquals(self.app.calls, 1)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Response Status'], '401 Unauthorized')
-        self.assertEquals(resp_data['Errors'], [])
+        self.assertEqual(self.app.calls, 1)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Response Status'], '401 Unauthorized')
+        self.assertEqual(resp_data['Errors'], [])
 
     def test_extract_tar_fail_obj_401(self):
         self.build_tar()
@@ -344,10 +447,10 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Response Status'], '401 Unauthorized')
-        self.assertEquals(
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Response Status'], '401 Unauthorized')
+        self.assertEqual(
             resp_data['Errors'],
             [['cont/base_fails1/sub_dir1/sub1_file1', '401 Unauthorized']])
 
@@ -359,10 +462,10 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        self.assertEquals(self.app.calls, 6)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Files Created'], 4)
-        self.assertEquals(
+        self.assertEqual(self.app.calls, 6)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Files Created'], 4)
+        self.assertEqual(
             resp_data['Errors'],
             [['cont/base_fails1/' + ('f' * 101), '400 Bad Request']])
 
@@ -374,10 +477,10 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, 'gz')
-        self.assertEquals(self.app.calls, 0)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-        self.assertEquals(
+        self.assertEqual(self.app.calls, 0)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+        self.assertEqual(
             resp_data['Response Body'].lower(),
             'invalid tar file: not a gzip file')
 
@@ -391,14 +494,14 @@ class TestUntar(unittest.TestCase):
                                                           'tar_fails.tar'))
             req.headers['transfer-encoding'] = 'chunked'
             resp_body = self.handle_extract_and_iter(req, '')
-            self.assertEquals(self.app.calls, 5)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Number Files Created'], 3)
-            self.assertEquals(
+            self.assertEqual(self.app.calls, 5)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Number Files Created'], 3)
+            self.assertEqual(
                 resp_data['Errors'],
                 [['cont/base_fails1/' + ('f' * 101), '400 Bad Request']])
 
-    @patch.object(bulk, 'MAX_FILE_SIZE', 4)
+    @patch.object(constraints, 'MAX_FILE_SIZE', 4)
     def test_extract_tar_fail_max_file_size(self):
         tar = self.build_tar()
         dir_tree = [{'test': [{'sub_dir1': ['sub1_file1']}]}]
@@ -415,8 +518,8 @@ class TestUntar(unittest.TestCase):
             os.path.join(self.testdir, 'tar_works.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        resp_data = json.loads(resp_body)
-        self.assertEquals(
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(
             resp_data['Errors'],
             [['cont' + self.testdir + '/test/sub_dir1/sub1_file1',
               '413 Request Entity Too Large']])
@@ -434,10 +537,10 @@ class TestUntar(unittest.TestCase):
                                 headers={'Accept': 'application/json'})
             req.headers['transfer-encoding'] = 'chunked'
             resp_body = self.handle_extract_and_iter(req, '')
-            self.assertEquals(self.app.calls, 5)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-            self.assertEquals(
+            self.assertEqual(self.app.calls, 5)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+            self.assertEqual(
                 resp_data['Response Body'],
                 'More than 1 containers to create from tar.')
 
@@ -453,9 +556,9 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        resp_data = json.loads(resp_body)
-        self.assertEquals(self.app.calls, 5)
-        self.assertEquals(len(resp_data['Errors']), 5)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(self.app.calls, 5)
+        self.assertEqual(len(resp_data['Errors']), 5)
 
     def test_extract_tar_fail_create_cont_value_err(self):
         self.build_tar()
@@ -470,10 +573,10 @@ class TestUntar(unittest.TestCase):
 
         with patch.object(self.bulk, 'create_container', bad_create):
             resp_body = self.handle_extract_and_iter(req, '')
-            resp_data = json.loads(resp_body)
-            self.assertEquals(self.app.calls, 0)
-            self.assertEquals(len(resp_data['Errors']), 5)
-            self.assertEquals(
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(self.app.calls, 0)
+            self.assertEqual(len(resp_data['Errors']), 5)
+            self.assertEqual(
                 resp_data['Errors'][0],
                 ['cont/base_fails1/sub_dir1/sub1_file1', '400 Bad Request'])
 
@@ -488,11 +591,11 @@ class TestUntar(unittest.TestCase):
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
         resp_body = self.handle_extract_and_iter(req, '')
-        resp_data = json.loads(resp_body)
-        self.assertEquals(self.app.calls, 4)
-        self.assertEquals(resp_data['Number Files Created'], 2)
-        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-        self.assertEquals(
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(self.app.calls, 4)
+        self.assertEqual(resp_data['Number Files Created'], 2)
+        self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+        self.assertEqual(
             resp_data['Errors'],
             [['sub_dir2/sub2%DEfile1', '412 Precondition Failed'],
              ['sub_%DEdir3/sub4_dir1/sub4_file1', '412 Precondition Failed']])
@@ -500,10 +603,10 @@ class TestUntar(unittest.TestCase):
     def test_get_response_body(self):
         txt_body = bulk.get_response_body(
             'bad_formay', {'hey': 'there'}, [['json > xml', '202 Accepted']])
-        self.assert_('hey: there' in txt_body)
+        self.assertTrue('hey: there' in txt_body)
         xml_body = bulk.get_response_body(
             'text/xml', {'hey': 'there'}, [['json > xml', '202 Accepted']])
-        self.assert_('&gt' in xml_body)
+        self.assertTrue('&gt' in xml_body)
 
 
 class TestDelete(unittest.TestCase):
@@ -533,42 +636,42 @@ class TestDelete(unittest.TestCase):
         resp_body = ''.join(self.bulk.handle_delete_iter(
             req, objs_to_delete=objs_to_delete,
             out_content_type='application/json'))
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths, ['/delete_works/AUTH_Acc/c/file_a',
                                     '/delete_works/AUTH_Acc/c/file_d'])
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-        self.assertEquals(resp_data['Number Deleted'], 2)
-        self.assertEquals(resp_data['Number Not Found'], 1)
-        self.assertEquals(resp_data['Errors'],
-                          [['/c/file_c', 'unauthorized']])
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+        self.assertEqual(resp_data['Number Deleted'], 2)
+        self.assertEqual(resp_data['Number Not Found'], 1)
+        self.assertEqual(resp_data['Errors'],
+                         [['/c/file_c', 'unauthorized']])
 
     def test_bulk_delete_works_with_POST_verb(self):
         req = Request.blank('/delete_works/AUTH_Acc', body='/c/f\n/c/f404',
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/f', '/delete_works/AUTH_Acc/c/f404'])
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Deleted'], 1)
-        self.assertEquals(resp_data['Number Not Found'], 1)
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Deleted'], 1)
+        self.assertEqual(resp_data['Number Not Found'], 1)
 
     def test_bulk_delete_works_with_DELETE_verb(self):
         req = Request.blank('/delete_works/AUTH_Acc', body='/c/f\n/c/f404',
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
         resp_body = self.handle_delete_and_iter(req)
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/f', '/delete_works/AUTH_Acc/c/f404'])
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Deleted'], 1)
-        self.assertEquals(resp_data['Number Not Found'], 1)
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Deleted'], 1)
+        self.assertEqual(resp_data['Number Not Found'], 1)
 
     def test_bulk_delete_bad_content_type(self):
         req = Request.blank('/delete_works/AUTH_Acc',
@@ -578,42 +681,42 @@ class TestDelete(unittest.TestCase):
                             headers={'Accept': 'application/json',
                                      'Content-Type': 'text/xml'})
         req.method = 'POST'
-        req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
+        req.environ['wsgi.input'] = BytesIO(b'/c/f\n/c/f404')
         resp_body = self.handle_delete_and_iter(req)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Response Status'], '406 Not Acceptable')
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Response Status'], '406 Not Acceptable')
 
     def test_bulk_delete_call_and_content_type(self):
         def fake_start_response(*args, **kwargs):
-            self.assertEquals(args[1][0], ('Content-Type', 'application/json'))
+            self.assertEqual(args[1][0], ('Content-Type', 'application/json'))
 
         req = Request.blank('/delete_works/AUTH_Acc?bulk-delete')
         req.method = 'POST'
         req.headers['Transfer-Encoding'] = 'chunked'
         req.headers['Accept'] = 'application/json'
-        req.environ['wsgi.input'] = StringIO('/c/f%20')
+        req.environ['wsgi.input'] = BytesIO(b'/c/f%20')
         list(self.bulk(req.environ, fake_start_response))  # iterate over resp
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths, ['/delete_works/AUTH_Acc/c/f '])
-        self.assertEquals(self.app.calls, 1)
+        self.assertEqual(self.app.calls, 1)
 
     def test_bulk_delete_get_objs(self):
         req = Request.blank('/delete_works/AUTH_Acc', body='1%20\r\n2\r\n')
         req.method = 'POST'
         with patch.object(self.bulk, 'max_deletes_per_request', 2):
             results = self.bulk.get_objs_to_delete(req)
-            self.assertEquals(results, [{'name': '1 '}, {'name': '2'}])
+            self.assertEqual(results, [{'name': '1 '}, {'name': '2'}])
 
-        with patch.object(bulk, 'MAX_PATH_LENGTH', 2):
+        with patch.object(self.bulk, 'max_path_length', 2):
             results = []
-            req.environ['wsgi.input'] = StringIO('1\n2\n3')
+            req.environ['wsgi.input'] = BytesIO(b'1\n2\n3')
             results = self.bulk.get_objs_to_delete(req)
-            self.assertEquals(results,
-                              [{'name': '1'}, {'name': '2'}, {'name': '3'}])
+            self.assertEqual(results,
+                             [{'name': '1'}, {'name': '2'}, {'name': '3'}])
 
         with patch.object(self.bulk, 'max_deletes_per_request', 9):
-            with patch.object(bulk, 'MAX_PATH_LENGTH', 1):
-                req_body = '\n'.join([str(i) for i in xrange(10)])
+            with patch.object(self.bulk, 'max_path_length', 1):
+                req_body = '\n'.join([str(i) for i in range(10)])
                 req = Request.blank('/delete_works/AUTH_Acc', body=req_body)
                 self.assertRaises(
                     HTTPException, self.bulk.get_objs_to_delete, req)
@@ -624,21 +727,21 @@ class TestDelete(unittest.TestCase):
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/f',
              '/delete_works/AUTH_Acc/c/f404',
              '/delete_works/AUTH_Acc/c/%25'])
-        self.assertEquals(self.app.calls, 3)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Deleted'], 2)
-        self.assertEquals(resp_data['Number Not Found'], 1)
+        self.assertEqual(self.app.calls, 3)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Deleted'], 2)
+        self.assertEqual(resp_data['Number Not Found'], 1)
 
     def test_bulk_delete_too_many_newlines(self):
         req = Request.blank('/delete_works/AUTH_Acc')
         req.method = 'POST'
-        data = '\n\n' * self.bulk.max_deletes_per_request
-        req.environ['wsgi.input'] = StringIO(data)
+        data = b'\n\n' * self.bulk.max_deletes_per_request
+        req.environ['wsgi.input'] = BytesIO(data)
         req.content_length = len(data)
         resp_body = self.handle_delete_and_iter(req)
         self.assertTrue('413 Request Entity Too Large' in resp_body)
@@ -651,19 +754,20 @@ class TestDelete(unittest.TestCase):
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
-        self.assertEquals(
+        self.assertEqual(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/ obj \xe2\x99\xa1',
              '/delete_works/AUTH_Acc/c/ objbadutf8'])
 
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Deleted'], 1)
-        self.assertEquals(len(resp_data['Errors']), 2)
-        self.assertEquals(
-            resp_data['Errors'],
-            [[urllib.quote('c/ objbadutf8'), '412 Precondition Failed'],
-             [urllib.quote('/c/f\xdebadutf8'), '412 Precondition Failed']])
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Deleted'], 1)
+        self.assertEqual(len(resp_data['Errors']), 2)
+        self.assertEqual(resp_data['Errors'],
+                         [[urllib.parse.quote('c/ objbadutf8'),
+                           '412 Precondition Failed'],
+                          [urllib.parse.quote('/c/f\xdebadutf8'),
+                           '412 Precondition Failed']])
 
     def test_bulk_delete_no_body(self):
         req = Request.blank('/unauth/AUTH_acc/')
@@ -680,22 +784,22 @@ class TestDelete(unittest.TestCase):
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
-        self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Errors'], [['/c/f', '401 Unauthorized']])
-        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-        self.assertEquals(resp_data['Number Deleted'], 1)
+        self.assertEqual(self.app.calls, 2)
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Errors'], [['/c/f', '401 Unauthorized']])
+        self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+        self.assertEqual(resp_data['Number Deleted'], 1)
 
     def test_bulk_delete_500_resp(self):
         req = Request.blank('/broke/AUTH_acc/', body='/c/f\nc/f2\n',
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(
             resp_data['Errors'],
             [['/c/f', '500 Internal Error'], ['c/f2', '500 Internal Error']])
-        self.assertEquals(resp_data['Response Status'], '502 Bad Gateway')
+        self.assertEqual(resp_data['Response Status'], '502 Bad Gateway')
 
     def test_bulk_delete_bad_path(self):
         req = Request.blank('/delete_cont_fail/')
@@ -710,11 +814,11 @@ class TestDelete(unittest.TestCase):
                    new=mock.MagicMock(wraps=sleep,
                                       return_value=None)) as mock_sleep:
             resp_body = self.handle_delete_and_iter(req)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Number Deleted'], 0)
-            self.assertEquals(resp_data['Errors'], [['c', '409 Conflict']])
-            self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-            self.assertEquals([], mock_sleep.call_args_list)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Number Deleted'], 0)
+            self.assertEqual(resp_data['Errors'], [['c', '409 Conflict']])
+            self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+            self.assertEqual([], mock_sleep.call_args_list)
 
     def test_bulk_delete_container_delete_retry_and_fails(self):
         self.bulk.retry_count = 3
@@ -725,14 +829,14 @@ class TestDelete(unittest.TestCase):
                    new=mock.MagicMock(wraps=sleep,
                                       return_value=None)) as mock_sleep:
             resp_body = self.handle_delete_and_iter(req)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Number Deleted'], 0)
-            self.assertEquals(resp_data['Errors'], [['c', '409 Conflict']])
-            self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-            self.assertEquals([call(self.bulk.retry_interval),
-                               call(self.bulk.retry_interval ** 2),
-                               call(self.bulk.retry_interval ** 3)],
-                              mock_sleep.call_args_list)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Number Deleted'], 0)
+            self.assertEqual(resp_data['Errors'], [['c', '409 Conflict']])
+            self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+            self.assertEqual([call(self.bulk.retry_interval),
+                              call(self.bulk.retry_interval ** 2),
+                              call(self.bulk.retry_interval ** 3)],
+                             mock_sleep.call_args_list)
 
     def test_bulk_delete_container_delete_retry_and_success(self):
         self.bulk.retry_count = 3
@@ -744,30 +848,30 @@ class TestDelete(unittest.TestCase):
                    new=mock.MagicMock(wraps=sleep,
                                       return_value=None)) as mock_sleep:
             resp_body = self.handle_delete_and_iter(req)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Number Deleted'], 1)
-            self.assertEquals(resp_data['Errors'], [])
-            self.assertEquals(resp_data['Response Status'], '200 OK')
-            self.assertEquals([call(self.bulk.retry_interval),
-                               call(self.bulk.retry_interval ** 2)],
-                              mock_sleep.call_args_list)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Number Deleted'], 1)
+            self.assertEqual(resp_data['Errors'], [])
+            self.assertEqual(resp_data['Response Status'], '200 OK')
+            self.assertEqual([call(self.bulk.retry_interval),
+                              call(self.bulk.retry_interval ** 2)],
+                             mock_sleep.call_args_list)
 
     def test_bulk_delete_bad_file_too_long(self):
         req = Request.blank('/delete_works/AUTH_Acc',
                             headers={'Accept': 'application/json'})
         req.method = 'POST'
-        bad_file = 'c/' + ('1' * bulk.MAX_PATH_LENGTH)
-        data = '/c/f\n' + bad_file + '\n/c/f'
-        req.environ['wsgi.input'] = StringIO(data)
+        bad_file = 'c/' + ('1' * self.bulk.max_path_length)
+        data = b'/c/f\n' + bad_file.encode('ascii') + b'\n/c/f'
+        req.environ['wsgi.input'] = BytesIO(data)
         req.headers['Transfer-Encoding'] = 'chunked'
         resp_body = self.handle_delete_and_iter(req)
-        resp_data = json.loads(resp_body)
-        self.assertEquals(resp_data['Number Deleted'], 2)
-        self.assertEquals(resp_data['Errors'], [[bad_file, '400 Bad Request']])
-        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
+        resp_data = utils.json.loads(resp_body)
+        self.assertEqual(resp_data['Number Deleted'], 2)
+        self.assertEqual(resp_data['Errors'], [[bad_file, '400 Bad Request']])
+        self.assertEqual(resp_data['Response Status'], '400 Bad Request')
 
     def test_bulk_delete_bad_file_over_twice_max_length(self):
-        body = '/c/f\nc/' + ('123456' * bulk.MAX_PATH_LENGTH) + '\n'
+        body = '/c/f\nc/' + ('123456' * self.bulk.max_path_length) + '\n'
         req = Request.blank('/delete_works/AUTH_Acc', body=body)
         req.method = 'POST'
         resp_body = self.handle_delete_and_iter(req)
@@ -779,14 +883,14 @@ class TestDelete(unittest.TestCase):
         req.method = 'POST'
         with patch.object(self.bulk, 'max_failed_deletes', 2):
             resp_body = self.handle_delete_and_iter(req)
-            self.assertEquals(self.app.calls, 2)
-            resp_data = json.loads(resp_body)
-            self.assertEquals(resp_data['Response Status'], '400 Bad Request')
-            self.assertEquals(resp_data['Response Body'],
-                              'Max delete failures exceeded')
-            self.assertEquals(resp_data['Errors'],
-                              [['/c/f1', '401 Unauthorized'],
-                               ['/c/f2', '401 Unauthorized']])
+            self.assertEqual(self.app.calls, 2)
+            resp_data = utils.json.loads(resp_body)
+            self.assertEqual(resp_data['Response Status'], '400 Bad Request')
+            self.assertEqual(resp_data['Response Body'],
+                             'Max delete failures exceeded')
+            self.assertEqual(resp_data['Errors'],
+                             [['/c/f1', '401 Unauthorized'],
+                              ['/c/f2', '401 Unauthorized']])
 
 
 class TestSwiftInfo(unittest.TestCase):
