@@ -205,7 +205,7 @@ class SwiftAuth(unittest.TestCase):
         req = self._make_request('/v1/AUTH_account',
                                  environ={'swift.authorize_override': True})
         resp = req.get_response(self.test_auth)
-        self.assertEquals(resp.status_int, 401)
+        self.assertEqual(resp.status_int, 401)
 
     def test_override_asked_for_and_allowed(self):
         conf = {'allow_overrides': 'true'}
@@ -213,13 +213,13 @@ class SwiftAuth(unittest.TestCase):
         req = self._make_request('/v1/AUTH_account',
                                  environ={'swift.authorize_override': True})
         resp = req.get_response(self.test_auth)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_override_default_allowed(self):
         req = self._make_request('/v1/AUTH_account',
                                  environ={'swift.authorize_override': True})
         resp = req.get_response(self.test_auth)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_anonymous_options_allowed(self):
         req = self._make_request('/v1/AUTH_account',
@@ -582,7 +582,7 @@ class BaseTestAuthorize(unittest.TestCase):
                                  user_domain_id)
         token_info = _fake_token_info(version=auth_version)
         env.update({'keystone.token_info': token_info})
-        return self.test_auth._integral_keystone_identity(env)
+        return self.test_auth._keystone_identity(env)
 
 
 class TestAuthorize(BaseTestAuthorize):
@@ -604,14 +604,14 @@ class TestAuthorize(BaseTestAuthorize):
         req = self._make_request(path, headers=headers, environ=default_env)
         req.acl = acl
 
-        env_identity = self.test_auth._integral_keystone_identity(req.environ)
+        env_identity = self.test_auth._keystone_identity(req.environ)
         result = self.test_auth.authorize(env_identity, req)
 
         # if we have requested an exception but nothing came back then
         if exception and not result:
             self.fail("error %s was not returned" % (str(exception)))
         elif exception:
-            self.assertEquals(result.status_int, exception)
+            self.assertEqual(result.status_int, exception)
         else:
             self.assertTrue(result is None)
         return req
@@ -912,25 +912,6 @@ class TestAuthorize(BaseTestAuthorize):
         self._check_authenticate(acl=acl, identity=id, env=env)
 
     def test_keystone_identity(self):
-        user_name = 'U_NAME'
-        project = ('P_ID', 'P_NAME')
-        roles = ('ROLE1', 'ROLE2')
-
-        req = Request.blank('/v/a/c/o')
-        req.headers.update({'X-Identity-Status': 'Confirmed',
-                            'X-Roles': ' %s , %s ' % roles,
-                            'X-User-Name': user_name,
-                            'X-Tenant-Id': project[0],
-                            'X-Tenant-Name': project[1]})
-
-        expected = {'user': user_name,
-                    'tenant': project,
-                    'roles': list(roles)}
-        data = self.test_auth._keystone_identity(req.environ)
-
-        self.assertEquals(expected, data)
-
-    def test_integral_keystone_identity(self):
         user = ('U_ID', 'U_NAME')
         roles = ('ROLE1', 'ROLE2')
         service_roles = ('ROLE3', 'ROLE4')
@@ -940,7 +921,7 @@ class TestAuthorize(BaseTestAuthorize):
 
         # no valid identity info in headers
         req = Request.blank('/v/a/c/o')
-        data = self.test_auth._integral_keystone_identity(req.environ)
+        data = self.test_auth._keystone_identity(req.environ)
         self.assertEqual(None, data)
 
         # valid identity info in headers, but status unconfirmed
@@ -954,7 +935,7 @@ class TestAuthorize(BaseTestAuthorize):
                             'X-User-Domain-Name': user_domain[1],
                             'X-Project-Domain-Id': project_domain[0],
                             'X-Project-Domain-Name': project_domain[1]})
-        data = self.test_auth._integral_keystone_identity(req.environ)
+        data = self.test_auth._keystone_identity(req.environ)
         self.assertEqual(None, data)
 
         # valid identity info in headers, no token info in environ
@@ -966,8 +947,8 @@ class TestAuthorize(BaseTestAuthorize):
                     'user_domain': (None, None),
                     'project_domain': (None, None),
                     'auth_version': 0}
-        data = self.test_auth._integral_keystone_identity(req.environ)
-        self.assertEquals(expected, data)
+        data = self.test_auth._keystone_identity(req.environ)
+        self.assertEqual(expected, data)
 
         # v2 token info in environ
         req.environ['keystone.token_info'] = _fake_token_info(version='2')
@@ -978,8 +959,8 @@ class TestAuthorize(BaseTestAuthorize):
                     'user_domain': (None, None),
                     'project_domain': (None, None),
                     'auth_version': 2}
-        data = self.test_auth._integral_keystone_identity(req.environ)
-        self.assertEquals(expected, data)
+        data = self.test_auth._keystone_identity(req.environ)
+        self.assertEqual(expected, data)
 
         # v3 token info in environ
         req.environ['keystone.token_info'] = _fake_token_info(version='3')
@@ -990,8 +971,8 @@ class TestAuthorize(BaseTestAuthorize):
                     'user_domain': user_domain,
                     'project_domain': project_domain,
                     'auth_version': 3}
-        data = self.test_auth._integral_keystone_identity(req.environ)
-        self.assertEquals(expected, data)
+        data = self.test_auth._keystone_identity(req.environ)
+        self.assertEqual(expected, data)
 
         # service token in environ
         req.headers.update({'X-Service-Roles': '%s,%s' % service_roles})
@@ -1002,8 +983,8 @@ class TestAuthorize(BaseTestAuthorize):
                     'user_domain': user_domain,
                     'project_domain': project_domain,
                     'auth_version': 3}
-        data = self.test_auth._integral_keystone_identity(req.environ)
-        self.assertEquals(expected, data)
+        data = self.test_auth._keystone_identity(req.environ)
+        self.assertEqual(expected, data)
 
     def test_get_project_domain_id(self):
         sysmeta = {}
