@@ -15,8 +15,9 @@
 
 import json
 import unittest
+import mock
 
-from swift.common.swob import Request, Response
+from swift.common.swob import Request, Response, HTTPUnauthorized
 from swift.common.middleware import staticweb
 
 
@@ -36,7 +37,8 @@ meta_map = {
                     'web-error': 'error.html',
                     'web-listings': 't',
                     'web-listings-css': 'listing.css'}},
-    'c6': {'meta': {'web-listings': 't'}},
+    'c6': {'meta': {'web-listings': 't',
+                    'web-error': 'error.html'}},
     'c7': {'meta': {'web-listings': 'f'}},
     'c8': {'meta': {'web-error': 'error.html',
                     'web-listings': 't',
@@ -73,6 +75,10 @@ class FakeApp(object):
 
     def __call__(self, env, start_response):
         self.calls += 1
+        if 'swift.authorize' in env:
+            resp = env['swift.authorize'](Request(env))
+            if resp:
+                return resp(env, start_response)
         if env['PATH_INFO'] == '/':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1':
@@ -182,6 +188,14 @@ class FakeApp(object):
             return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c6/subdir':
             return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c6/401error.html':
+            return Response(status='200 Ok', body='''
+<html>
+    <body style="background: #000000; color: #ffaaaa">
+        <p>Hey, you're not authorized to see this!</p>
+    </body>
+</html>
+            '''.strip())(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c7', '/v1/a/c7/'):
             return self.listing(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c8', '/v1/a/c8/'):
@@ -379,11 +393,30 @@ class FakeApp(object):
                         body=body)(env, start_response)
 
 
+class FakeAuthFilter(object):
+
+    def __init__(self, app, deny_objects=False, deny_listing=False):
+        self.app = app
+        self.deny_objects = deny_objects
+        self.deny_listing = deny_listing
+
+    def authorize(self, req):
+        path_parts = req.path.strip('/').split('/')
+        if ((self.deny_objects and len(path_parts) > 3)
+                or (self.deny_listing and len(path_parts) == 3)):
+            return HTTPUnauthorized()
+
+    def __call__(self, env, start_response):
+        env['swift.authorize'] = self.authorize
+        return self.app(env, start_response)
+
+
 class TestStaticWeb(unittest.TestCase):
 
     def setUp(self):
         self.app = FakeApp()
-        self.test_staticweb = staticweb.filter_factory({})(self.app)
+        self.test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app))
         self._orig_get_container_info = staticweb.get_container_info
         staticweb.get_container_info = mock_get_container_info
 
@@ -393,117 +426,117 @@ class TestStaticWeb(unittest.TestCase):
     def test_app_set(self):
         app = FakeApp()
         sw = staticweb.filter_factory({})(app)
-        self.assertEquals(sw.app, app)
+        self.assertEqual(sw.app, app)
 
     def test_conf_set(self):
         conf = {'blah': 1}
         sw = staticweb.filter_factory(conf)(FakeApp())
-        self.assertEquals(sw.conf, conf)
+        self.assertEqual(sw.conf, conf)
 
     def test_root(self):
         resp = Request.blank('/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_version(self):
         resp = Request.blank('/v1').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 412)
+        self.assertEqual(resp.status_int, 412)
 
     def test_account(self):
         resp = Request.blank('/v1/a').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 401)
+        self.assertEqual(resp.status_int, 401)
 
     def test_container1(self):
         resp = Request.blank('/v1/a/c1').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 401)
+        self.assertEqual(resp.status_int, 401)
 
     def test_container1_web_mode_explicitly_off(self):
         resp = Request.blank('/v1/a/c1',
                              headers={'x-web-mode': 'false'}).get_response(
                                  self.test_staticweb)
-        self.assertEquals(resp.status_int, 401)
+        self.assertEqual(resp.status_int, 401)
 
     def test_container1_web_mode_explicitly_on(self):
         resp = Request.blank('/v1/a/c1',
                              headers={'x-web-mode': 'true'}).get_response(
                                  self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container2(self):
         resp = Request.blank('/v1/a/c2').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.content_type, 'text/plain')
-        self.assertEquals(len(resp.body.split('\n')),
-                          int(resp.headers['x-container-object-count']))
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.content_type, 'text/plain')
+        self.assertEqual(len(resp.body.split('\n')),
+                         int(resp.headers['x-container-object-count']))
 
     def test_container2_web_mode_explicitly_off(self):
         resp = Request.blank(
             '/v1/a/c2',
             headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.content_type, 'text/plain')
-        self.assertEquals(len(resp.body.split('\n')),
-                          int(resp.headers['x-container-object-count']))
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.content_type, 'text/plain')
+        self.assertEqual(len(resp.body.split('\n')),
+                         int(resp.headers['x-container-object-count']))
 
     def test_container2_web_mode_explicitly_on(self):
         resp = Request.blank(
             '/v1/a/c2',
             headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container2onetxt(self):
         resp = Request.blank(
             '/v1/a/c2/one.txt').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container2json(self):
         resp = Request.blank(
             '/v1/a/c2?format=json').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.content_type, 'application/json')
-        self.assertEquals(len(json.loads(resp.body)),
-                          int(resp.headers['x-container-object-count']))
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.content_type, 'application/json')
+        self.assertEqual(len(json.loads(resp.body)),
+                         int(resp.headers['x-container-object-count']))
 
     def test_container2json_web_mode_explicitly_off(self):
         resp = Request.blank(
             '/v1/a/c2?format=json',
             headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.content_type, 'application/json')
-        self.assertEquals(len(json.loads(resp.body)),
-                          int(resp.headers['x-container-object-count']))
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.content_type, 'application/json')
+        self.assertEqual(len(json.loads(resp.body)),
+                         int(resp.headers['x-container-object-count']))
 
     def test_container2json_web_mode_explicitly_on(self):
         resp = Request.blank(
             '/v1/a/c2?format=json',
             headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container3(self):
         resp = Request.blank('/v1/a/c3').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
-        self.assertEquals(resp.headers['location'],
-                          'http://localhost/v1/a/c3/')
+        self.assertEqual(resp.status_int, 301)
+        self.assertEqual(resp.headers['location'],
+                         'http://localhost/v1/a/c3/')
 
     def test_container3indexhtml(self):
         resp = Request.blank('/v1/a/c3/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Test main index.html file.' in resp.body)
 
     def test_container3subsubdir(self):
         resp = Request.blank(
             '/v1/a/c3/subdir3/subsubdir').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
+        self.assertEqual(resp.status_int, 301)
 
     def test_container3subsubdircontents(self):
         resp = Request.blank(
             '/v1/a/c3/subdir3/subsubdir/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.body, 'index file')
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.body, 'index file')
 
     def test_container3subdir(self):
         resp = Request.blank(
             '/v1/a/c3/subdir/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c3/subdir/' in resp.body)
         self.assertTrue('</style>' in resp.body)
         self.assertTrue('<link' not in resp.body)
@@ -512,100 +545,121 @@ class TestStaticWeb(unittest.TestCase):
     def test_container3subdirx(self):
         resp = Request.blank(
             '/v1/a/c3/subdirx/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container3subdiry(self):
         resp = Request.blank(
             '/v1/a/c3/subdiry/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
 
     def test_container3subdirz(self):
         resp = Request.blank(
             '/v1/a/c3/subdirz').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
+        self.assertEqual(resp.status_int, 301)
 
     def test_container3unknown(self):
         resp = Request.blank(
             '/v1/a/c3/unknown').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue("Chrome's 404 fancy-page sucks." not in resp.body)
 
     def test_container3bindexhtml(self):
         resp = Request.blank('/v1/a/c3b/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 204)
-        self.assertEquals(resp.body, '')
+        self.assertEqual(resp.status_int, 204)
+        self.assertEqual(resp.body, '')
 
     def test_container4indexhtml(self):
         resp = Request.blank('/v1/a/c4/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c4/' in resp.body)
         self.assertTrue('href="listing.css"' in resp.body)
 
     def test_container4indexhtmlauthed(self):
         resp = Request.blank('/v1/a/c4').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
+        self.assertEqual(resp.status_int, 301)
         resp = Request.blank(
             '/v1/a/c4',
             environ={'REMOTE_USER': 'authed'}).get_response(
                 self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         resp = Request.blank(
             '/v1/a/c4', headers={'x-web-mode': 't'},
             environ={'REMOTE_USER': 'authed'}).get_response(
                 self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
+        self.assertEqual(resp.status_int, 301)
 
     def test_container4unknown(self):
         resp = Request.blank(
             '/v1/a/c4/unknown').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue("Chrome's 404 fancy-page sucks." in resp.body)
 
     def test_container4subdir(self):
         resp = Request.blank(
             '/v1/a/c4/subdir/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c4/subdir/' in resp.body)
         self.assertTrue('</style>' not in resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue('href="../listing.css"' in resp.body)
-        self.assertEquals(resp.headers['content-type'],
-                          'text/html; charset=UTF-8')
+        self.assertEqual(resp.headers['content-type'],
+                         'text/html; charset=UTF-8')
 
     def test_container4onetxt(self):
         resp = Request.blank(
             '/v1/a/c4/one.txt').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
 
     def test_container4twotxt(self):
         resp = Request.blank(
             '/v1/a/c4/two.txt').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 503)
+        self.assertEqual(resp.status_int, 503)
 
     def test_container5indexhtml(self):
         resp = Request.blank('/v1/a/c5/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 503)
+        self.assertEqual(resp.status_int, 503)
 
     def test_container5unknown(self):
         resp = Request.blank(
             '/v1/a/c5/unknown').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue("Chrome's 404 fancy-page sucks." not in resp.body)
 
     def test_container6subdir(self):
         resp = Request.blank(
             '/v1/a/c6/subdir').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
+        self.assertEqual(resp.status_int, 301)
+
+    def test_container6listing(self):
+        # container6 has web-listings = t, web-error=error.html
+        resp = Request.blank('/v1/a/c6/').get_response(self.test_staticweb)
+        self.assertEqual(resp.status_int, 200)
+
+        # expect custom 401 if request is not auth'd for listing but is auth'd
+        # to GET objects
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app), deny_listing=True)
+        resp = Request.blank('/v1/a/c6/').get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 401)
+        self.assertIn("Hey, you're not authorized to see this!", resp.body)
+
+        # expect default 401 if request is not auth'd for listing or object GET
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app), deny_listing=True,
+            deny_objects=True)
+        resp = Request.blank('/v1/a/c6/').get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 401)
+        self.assertNotIn("Hey, you're not authorized to see this!", resp.body)
 
     def test_container7listing(self):
         resp = Request.blank('/v1/a/c7/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue('Web Listing Disabled' in resp.body)
 
     def test_container8listingcss(self):
         resp = Request.blank(
             '/v1/a/c8/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c8/' in resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue(
@@ -614,7 +668,7 @@ class TestStaticWeb(unittest.TestCase):
     def test_container8subdirlistingcss(self):
         resp = Request.blank(
             '/v1/a/c8/subdir/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c8/subdir/' in resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue(
@@ -623,7 +677,7 @@ class TestStaticWeb(unittest.TestCase):
     def test_container9listingcss(self):
         resp = Request.blank(
             '/v1/a/c9/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c9/' in resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue('href="/absolute/listing.css"' in resp.body)
@@ -631,7 +685,7 @@ class TestStaticWeb(unittest.TestCase):
     def test_container9subdirlistingcss(self):
         resp = Request.blank(
             '/v1/a/c9/subdir/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c9/subdir/' in resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue('href="/absolute/listing.css"' in resp.body)
@@ -639,67 +693,102 @@ class TestStaticWeb(unittest.TestCase):
     def test_container10unicodesubdirlisting(self):
         resp = Request.blank(
             '/v1/a/c10/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c10/' in resp.body)
         resp = Request.blank(
             '/v1/a/c10/\xe2\x98\x83/').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c10/\xe2\x98\x83/' in resp.body)
         resp = Request.blank(
             '/v1/a/c10/\xe2\x98\x83/\xe2\x98\x83/'
         ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue(
             'Listing of /v1/a/c10/\xe2\x98\x83/\xe2\x98\x83/' in resp.body)
 
     def test_container11subdirmarkerobjectindex(self):
         resp = Request.blank('/v1/a/c11/subdir/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('<h2>c11 subdir index</h2>' in resp.body)
 
     def test_container11subdirmarkermatchdirtype(self):
         resp = Request.blank('/v1/a/c11a/subdir/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue('Index File Not Found' in resp.body)
 
     def test_container11subdirmarkeraltdirtype(self):
         resp = Request.blank('/v1/a/c11a/subdir2/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
 
     def test_container11subdirmarkerinvaliddirtype(self):
         resp = Request.blank('/v1/a/c11a/subdir3/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
 
     def test_container12unredirectedrequest(self):
         resp = Request.blank('/v1/a/c12/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
+        self.assertEqual(resp.status_int, 200)
         self.assertTrue('index file' in resp.body)
 
     def test_container_404_has_css(self):
         resp = Request.blank('/v1/a/c13/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue('listing.css' in resp.body)
 
     def test_container_404_has_no_css(self):
         resp = Request.blank('/v1/a/c7/').get_response(
             self.test_staticweb)
-        self.assertEquals(resp.status_int, 404)
+        self.assertEqual(resp.status_int, 404)
         self.assertTrue('listing.css' not in resp.body)
         self.assertTrue('<style' in resp.body)
 
     def test_subrequest_once_if_possible(self):
         resp = Request.blank(
             '/v1/a/c4/one.txt').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(resp.headers['x-object-meta-test'], 'value')
-        self.assertEquals(resp.body, '1')
-        self.assertEquals(self.app.calls, 1)
+        self.assertEqual(resp.status_int, 200)
+        self.assertEqual(resp.headers['x-object-meta-test'], 'value')
+        self.assertEqual(resp.body, '1')
+        self.assertEqual(self.app.calls, 1)
+
+    def test_no_auth_middleware(self):
+        resp = Request.blank('/v1/a/c3').get_response(self.test_staticweb)
+        self.assertEqual(resp.status_int, 301)
+        # Test without an authentication middleware before staticweb
+        # This is no longer handled by staticweb middleware, thus not returning
+        # a 301 redirect
+        self.test_staticweb = staticweb.filter_factory({})(self.app)
+        resp = Request.blank('/v1/a/c3').get_response(self.test_staticweb)
+        self.assertEqual(resp.status_int, 200)
+
+    def test_subrequest_not_override_auth(self):
+        app_call = \
+            'swift.common.middleware.staticweb._StaticWebContext._app_call'
+        orig_app_call = staticweb._StaticWebContext._app_call
+        _fail = self.fail
+
+        def hook_app_call(self, env):
+            if 'swift.authorize_override' in env:
+                _fail('staticweb must not create authorize info by itself')
+            return orig_app_call(self, env)
+
+        with mock.patch(app_call, hook_app_call):
+            # testing for _listing container
+            resp = Request.blank('/v1/a/c4/').get_response(self.test_staticweb)
+            self.assertEqual(resp.status_int, 200)  # sanity
+
+            # testing for _listing object subdir
+            resp = Request.blank(
+                '/v1/a/c4/unknown').get_response(self.test_staticweb)
+            self.assertEqual(resp.status_int, 404)
+
+            # testing for _error_response
+            resp = Request.blank('/v1/a/c5/').get_response(self.test_staticweb)
+            self.assertEqual(resp.status_int, 503)  # sanity
 
 
 if __name__ == '__main__':
