@@ -13,21 +13,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 import unittest
 import os
-import urllib
 from contextlib import contextmanager
-import StringIO
 from hashlib import md5
 import time
 
 import mock
+import six
+from six.moves import urllib
 
 from swift.common import direct_client
 from swift.common.exceptions import ClientException
-from swift.common.utils import json, Timestamp
+from swift.common.utils import Timestamp
 from swift.common.swob import HeaderKeyDict, RESPONSE_REASONS
-from swift.common.storage_policy import POLICY_INDEX, POLICIES
+from swift.common.storage_policy import POLICIES
+from six.moves.http_client import HTTPException
 
 from test.unit import patch_policies
 
@@ -67,8 +69,13 @@ class FakeConn(object):
     def getheaders(self):
         return self.resp_headers.items()
 
-    def read(self):
-        return self.body
+    def read(self, amt=None):
+        if isinstance(self.body, six.StringIO):
+            return self.body.read(amt)
+        elif amt is None:
+            return self.body
+        else:
+            return Exception('Not a StringIO entry')
 
     def send(self, data):
         if not self.etag:
@@ -96,13 +103,13 @@ class TestDirectClient(unittest.TestCase):
         self.account = u'\u062a account'
         self.container = u'\u062a container'
         self.obj = u'\u062a obj/name'
-        self.account_path = '/sda/0/%s' % urllib.quote(
+        self.account_path = '/sda/0/%s' % urllib.parse.quote(
             self.account.encode('utf-8'))
         self.container_path = '/sda/0/%s/%s' % tuple(
-            urllib.quote(p.encode('utf-8')) for p in (
+            urllib.parse.quote(p.encode('utf-8')) for p in (
                 self.account, self.container))
         self.obj_path = '/sda/0/%s/%s/%s' % tuple(
-            urllib.quote(p.encode('utf-8')) for p in (
+            urllib.parse.quote(p.encode('utf-8')) for p in (
                 self.account, self.container, self.obj))
         self.user_agent = 'direct-client %s' % os.getpid()
 
@@ -116,7 +123,7 @@ class TestDirectClient(unittest.TestCase):
         now = time.time()
         headers = direct_client.gen_headers(add_ts=True)
         self.assertEqual(headers['user-agent'], stub_user_agent)
-        self.assert_(now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
+        self.assertTrue(now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
         self.assertEqual(headers['x-timestamp'],
                          Timestamp(headers['x-timestamp']).internal)
         self.assertEqual(2, len(headers))
@@ -134,16 +141,18 @@ class TestDirectClient(unittest.TestCase):
             for add_ts in (True, False):
                 now = time.time()
                 headers = direct_client.gen_headers(
-                    {POLICY_INDEX: policy.idx}, add_ts=add_ts)
+                    {'X-Backend-Storage-Policy-Index': policy.idx},
+                    add_ts=add_ts)
                 self.assertEqual(headers['user-agent'], stub_user_agent)
-                self.assertEqual(headers[POLICY_INDEX], str(policy.idx))
+                self.assertEqual(headers['X-Backend-Storage-Policy-Index'],
+                                 str(policy.idx))
                 expected_header_count = 2
                 if add_ts:
                     expected_header_count += 1
                     self.assertEqual(
                         headers['x-timestamp'],
                         Timestamp(headers['x-timestamp']).internal)
-                    self.assert_(
+                    self.assertTrue(
                         now - 1 < Timestamp(headers['x-timestamp']) < now + 1)
                 self.assertEqual(expected_header_count, len(headers))
 
@@ -159,13 +168,19 @@ class TestDirectClient(unittest.TestCase):
 
         with mocked_http_conn(200, stub_headers, body) as conn:
             resp_headers, resp = direct_client.direct_get_account(
-                self.node, self.part, self.account)
+                self.node, self.part, self.account, marker='marker',
+                prefix='prefix', delimiter='delimiter', limit=1000)
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.account_path)
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
         self.assertEqual(resp_headers, stub_headers)
         self.assertEqual(json.loads(body), resp)
+        self.assertTrue('marker=marker' in conn.query_string)
+        self.assertTrue('delimiter=delimiter' in conn.query_string)
+        self.assertTrue('limit=1000' in conn.query_string)
+        self.assertTrue('prefix=prefix' in conn.query_string)
+        self.assertTrue('format=json' in conn.query_string)
 
     def test_direct_client_exception(self):
         stub_headers = {'X-Trans-Id': 'txb5f59485c578460f8be9e-0053478d09'}
@@ -185,7 +200,8 @@ class TestDirectClient(unittest.TestCase):
             'status 500',
         )
         for item in expected_err_msg_parts:
-            self.assert_(item in str(err), '%r was not in "%s"' % (item, err))
+            self.assertTrue(
+                item in str(err), '%r was not in "%s"' % (item, err))
         self.assertEqual(err.http_host, self.node['ip'])
         self.assertEqual(err.http_port, self.node['port'])
         self.assertEqual(err.http_device, self.node['device'])
@@ -222,7 +238,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.account_path)
         self.assertEqual(err.http_status, 500)
-        self.assert_('GET' in str(err))
+        self.assertTrue('GET' in str(err))
 
     def test_direct_delete_account(self):
         node = {'ip': '1.2.3.4', 'port': '6000', 'device': 'sda'}
@@ -239,7 +255,21 @@ class TestDirectClient(unittest.TestCase):
             path = args[3]
             self.assertEqual('/sda/0/a', path)
             headers = args[4]
-            self.assert_('X-Timestamp' in headers)
+            self.assertTrue('X-Timestamp' in headers)
+
+    def test_direct_delete_account_failure(self):
+        node = {'ip': '1.2.3.4', 'port': '6000', 'device': 'sda'}
+        part = '0'
+        account = 'a'
+
+        with mocked_http_conn(500) as conn:
+            try:
+                direct_client.direct_delete_account(node, part, account)
+            except ClientException as err:
+                pass
+            self.assertEqual('DELETE', conn.method)
+            self.assertEqual('/sda/0/a', conn.path)
+            self.assertEqual(err.http_status, 500)
 
     def test_direct_head_container(self):
         headers = HeaderKeyDict(key='value')
@@ -272,7 +302,7 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
         self.assertEqual(err.http_status, 503)
         self.assertEqual(err.http_headers, headers)
-        self.assert_('HEAD' in str(err))
+        self.assertTrue('HEAD' in str(err))
 
     def test_direct_head_container_deleted(self):
         important_timestamp = Timestamp(time.time()).internal
@@ -284,7 +314,7 @@ class TestDirectClient(unittest.TestCase):
                 direct_client.direct_head_container(
                     self.node, self.part, self.account, self.container)
             except Exception as err:
-                self.assert_(isinstance(err, ClientException))
+                self.assertTrue(isinstance(err, ClientException))
             else:
                 self.fail('ClientException not raised')
             self.assertEqual(conn.method, 'HEAD')
@@ -300,12 +330,19 @@ class TestDirectClient(unittest.TestCase):
 
         with mocked_http_conn(200, headers, body) as conn:
             resp_headers, resp = direct_client.direct_get_container(
-                self.node, self.part, self.account, self.container)
+                self.node, self.part, self.account, self.container,
+                marker='marker', prefix='prefix', delimiter='delimiter',
+                limit=1000)
 
         self.assertEqual(conn.req_headers['user-agent'],
                          'direct-client %s' % os.getpid())
         self.assertEqual(headers, resp_headers)
         self.assertEqual(json.loads(body), resp)
+        self.assertTrue('marker=marker' in conn.query_string)
+        self.assertTrue('delimiter=delimiter' in conn.query_string)
+        self.assertTrue('limit=1000' in conn.query_string)
+        self.assertTrue('prefix=prefix' in conn.query_string)
+        self.assertTrue('format=json' in conn.query_string)
 
     def test_direct_get_container_no_content_does_not_decode_body(self):
         headers = {}
@@ -326,6 +363,19 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.method, 'DELETE')
             self.assertEqual(conn.path, self.container_path)
 
+    def test_direct_delete_container_with_timestamp(self):
+        # ensure timestamp is different from any that might be auto-generated
+        timestamp = Timestamp(time.time() - 100)
+        headers = {'X-Timestamp': timestamp.internal}
+        with mocked_http_conn(200) as conn:
+            direct_client.direct_delete_container(
+                self.node, self.part, self.account, self.container,
+                headers=headers)
+            self.assertEqual(conn.method, 'DELETE')
+            self.assertEqual(conn.path, self.container_path)
+            self.assertTrue('X-Timestamp' in conn.req_headers)
+            self.assertEqual(timestamp, conn.req_headers['X-Timestamp'])
+
     def test_direct_delete_container_error(self):
         with mocked_http_conn(500) as conn:
             try:
@@ -340,7 +390,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.container_path)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('DELETE' in str(err))
+        self.assertTrue('DELETE' in str(err))
 
     def test_direct_put_container_object(self):
         headers = {'x-foo': 'bar'}
@@ -351,7 +401,7 @@ class TestDirectClient(unittest.TestCase):
                 headers=headers)
             self.assertEqual(conn.method, 'PUT')
             self.assertEqual(conn.path, self.obj_path)
-            self.assert_('x-timestamp' in conn.req_headers)
+            self.assertTrue('x-timestamp' in conn.req_headers)
             self.assertEqual('bar', conn.req_headers.get('x-foo'))
 
         self.assertEqual(rv, None)
@@ -371,7 +421,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('PUT' in str(err))
+        self.assertTrue('PUT' in str(err))
 
     def test_direct_delete_container_object(self):
         with mocked_http_conn(204) as conn:
@@ -397,7 +447,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('DELETE' in str(err))
+        self.assertTrue('DELETE' in str(err))
 
     def test_direct_head_object(self):
         headers = HeaderKeyDict({'x-foo': 'bar'})
@@ -411,8 +461,8 @@ class TestDirectClient(unittest.TestCase):
 
         self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
         self.assertEqual('bar', conn.req_headers.get('x-foo'))
-        self.assert_('x-timestamp' not in conn.req_headers,
-                     'x-timestamp was in HEAD request headers')
+        self.assertTrue('x-timestamp' not in conn.req_headers,
+                        'x-timestamp was in HEAD request headers')
         self.assertEqual(headers, resp)
 
     def test_direct_head_object_error(self):
@@ -429,7 +479,7 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('HEAD' in str(err))
+        self.assertTrue('HEAD' in str(err))
 
     def test_direct_head_object_not_found(self):
         important_timestamp = Timestamp(time.time()).internal
@@ -451,14 +501,14 @@ class TestDirectClient(unittest.TestCase):
                          important_timestamp)
 
     def test_direct_get_object(self):
-        contents = StringIO.StringIO('123456')
+        contents = six.StringIO('123456')
 
         with mocked_http_conn(200, body=contents) as conn:
             resp_header, obj_body = direct_client.direct_get_object(
                 self.node, self.part, self.account, self.container, self.obj)
             self.assertEqual(conn.method, 'GET')
             self.assertEqual(conn.path, self.obj_path)
-        self.assertEqual(obj_body, contents)
+        self.assertEqual(obj_body, contents.getvalue())
 
     def test_direct_get_object_error(self):
         with mocked_http_conn(500) as conn:
@@ -474,7 +524,25 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('GET' in str(err))
+        self.assertTrue('GET' in str(err))
+
+    def test_direct_get_object_chunks(self):
+        contents = six.StringIO('123456')
+        downloaded = b''
+
+        with mocked_http_conn(200, body=contents) as conn:
+            resp_header, obj_body = direct_client.direct_get_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                resp_chunk_size=2)
+            while obj_body:
+                try:
+                    chunk = obj_body.next()
+                except StopIteration:
+                    break
+                downloaded += chunk
+            self.assertEqual('GET', conn.method)
+            self.assertEqual(self.obj_path, conn.path)
+        self.assertEqual('123456', downloaded)
 
     def test_direct_post_object(self):
         headers = {'Key': 'value'}
@@ -508,10 +576,10 @@ class TestDirectClient(unittest.TestCase):
             for header in headers:
                 self.assertEqual(conn.req_headers[header], headers[header])
             self.assertEqual(conn.req_headers['user-agent'], self.user_agent)
-            self.assert_('x-timestamp' in conn.req_headers)
+            self.assertTrue('x-timestamp' in conn.req_headers)
 
         self.assertEqual(err.http_status, 500)
-        self.assert_('POST' in str(err))
+        self.assertTrue('POST' in str(err))
 
     def test_direct_delete_object(self):
         with mocked_http_conn(200) as conn:
@@ -521,6 +589,19 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
         self.assertEqual(resp, None)
 
+    def test_direct_delete_object_with_timestamp(self):
+        # ensure timestamp is different from any that might be auto-generated
+        timestamp = Timestamp(time.time() - 100)
+        headers = {'X-Timestamp': timestamp.internal}
+        with mocked_http_conn(200) as conn:
+            direct_client.direct_delete_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                headers=headers)
+            self.assertEqual(conn.method, 'DELETE')
+            self.assertEqual(conn.path, self.obj_path)
+            self.assertTrue('X-Timestamp' in conn.req_headers)
+            self.assertEqual(timestamp, conn.req_headers['X-Timestamp'])
+
     def test_direct_delete_object_error(self):
         with mocked_http_conn(503) as conn:
             try:
@@ -534,10 +615,10 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.method, 'DELETE')
             self.assertEqual(conn.path, self.obj_path)
         self.assertEqual(err.http_status, 503)
-        self.assert_('DELETE' in str(err))
+        self.assertTrue('DELETE' in str(err))
 
     def test_direct_put_object_with_content_length(self):
-        contents = StringIO.StringIO('123456')
+        contents = six.StringIO('123456')
 
         with mocked_http_conn(200) as conn:
             resp = direct_client.direct_put_object(
@@ -548,7 +629,7 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(md5('123456').hexdigest(), resp)
 
     def test_direct_put_object_fail(self):
-        contents = StringIO.StringIO('123456')
+        contents = six.StringIO('123456')
 
         with mocked_http_conn(500) as conn:
             try:
@@ -564,7 +645,7 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(err.http_status, 500)
 
     def test_direct_put_object_chunked(self):
-        contents = StringIO.StringIO('123456')
+        contents = six.StringIO('123456')
 
         with mocked_http_conn(200) as conn:
             resp = direct_client.direct_put_object(
@@ -574,6 +655,32 @@ class TestDirectClient(unittest.TestCase):
             self.assertEqual(conn.path, self.obj_path)
         self.assertEqual(md5('6\r\n123456\r\n0\r\n\r\n').hexdigest(), resp)
 
+    def test_direct_put_object_args(self):
+        # One test to cover all missing checks
+        contents = ""
+        with mocked_http_conn(200) as conn:
+            resp = direct_client.direct_put_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                contents, etag="testing-etag", content_type='Text')
+            self.assertEqual('PUT', conn.method)
+            self.assertEqual(self.obj_path, conn.path)
+            self.assertEqual(conn.req_headers['Content-Length'], '0')
+            self.assertEqual(conn.req_headers['Content-Type'], 'Text')
+        self.assertEqual(md5('0\r\n\r\n').hexdigest(), resp)
+
+    def test_direct_put_object_header_content_length(self):
+        contents = six.StringIO('123456')
+        stub_headers = HeaderKeyDict({
+            'Content-Length': '6'})
+
+        with mocked_http_conn(200) as conn:
+            resp = direct_client.direct_put_object(
+                self.node, self.part, self.account, self.container, self.obj,
+                contents, headers=stub_headers)
+            self.assertEqual('PUT', conn.method)
+            self.assertEqual(conn.req_headers['Content-length'], '6')
+        self.assertEqual(md5('123456').hexdigest(), resp)
+
     def test_retry(self):
         headers = HeaderKeyDict({'key': 'value'})
 
@@ -587,6 +694,48 @@ class TestDirectClient(unittest.TestCase):
         self.assertEqual(headers, resp)
         self.assertEqual(attempts, 1)
 
+    def test_retry_client_exception(self):
+        err_log_file = six.StringIO()
+
+        def mock_err_logger(err):
+            err_log_file.write(err)
+
+        with mocked_http_conn(500) as conn:
+            try:
+                attempts, resp = direct_client.retry(
+                    direct_client.direct_delete_object, self.node, self.part,
+                    self.account, self.container, self.obj, retries=2,
+                    error_log=mock_err_logger)
+            except ClientException as err:
+                pass
+            self.assertEqual('DELETE', conn.method)
+            self.assertTrue(err_log_file.len)
+            self.assertEqual(err.http_status, 500)
+        err_log_file.close()
+
+    def test_retry_http_exception(self):
+        err_log_file = six.StringIO()
+
+        def mock_err_logger(err):
+            err_log_file.write(err)
+
+        def mock_direct_delete_object(node, part, account, container, obj,
+                                      conn_timeout=5, response_timeout=15,
+                                      headers=None):
+            resp = "Unable to delete object"
+            raise HTTPException('Object', 'DELETE', resp)
+
+        with mocked_http_conn(500):
+            with mock.patch('swift.common.direct_client.direct_delete_object',
+                            mock_direct_delete_object):
+                try:
+                    attempts, resp = direct_client.retry(
+                        direct_client.direct_delete_object, self.node,
+                        self.part, self.account, self.container, self.obj,
+                        retries=2, error_log=mock_err_logger)
+                except HTTPException:
+                    self.assertTrue(err_log_file.len)
+        err_log_file.close()
 
 if __name__ == '__main__':
     unittest.main()
