@@ -16,7 +16,6 @@
 import json
 import mock
 import unittest
-from urllib import quote
 import zlib
 from textwrap import dedent
 import os
@@ -24,10 +23,10 @@ import os
 import six
 from six import StringIO
 from six.moves import range
+from six.moves.urllib.parse import quote
 from test.unit import FakeLogger
 from eventlet.green import urllib2
-from swift.common import internal_client
-from swift.common import swob
+from swift.common import exceptions, internal_client, swob
 from swift.common.storage_policy import StoragePolicy
 
 from test.unit import with_tempdir, write_fake_ring, patch_policies
@@ -90,9 +89,9 @@ class GetMetadataInternalClient(internal_client.InternalClient):
     def _get_metadata(self, path, metadata_prefix, acceptable_statuses=None,
                       headers=None):
         self.get_metadata_called += 1
-        self.test.assertEquals(self.path, path)
-        self.test.assertEquals(self.metadata_prefix, metadata_prefix)
-        self.test.assertEquals(self.acceptable_statuses, acceptable_statuses)
+        self.test.assertEqual(self.path, path)
+        self.test.assertEqual(self.metadata_prefix, metadata_prefix)
+        self.test.assertEqual(self.acceptable_statuses, acceptable_statuses)
         return self.metadata
 
 
@@ -111,10 +110,10 @@ class SetMetadataInternalClient(internal_client.InternalClient):
             self, path, metadata, metadata_prefix='',
             acceptable_statuses=None):
         self.set_metadata_called += 1
-        self.test.assertEquals(self.path, path)
-        self.test.assertEquals(self.metadata_prefix, metadata_prefix)
-        self.test.assertEquals(self.metadata, metadata)
-        self.test.assertEquals(self.acceptable_statuses, acceptable_statuses)
+        self.test.assertEqual(self.path, path)
+        self.test.assertEqual(self.metadata_prefix, metadata_prefix)
+        self.test.assertEqual(self.metadata, metadata)
+        self.test.assertEqual(self.acceptable_statuses, acceptable_statuses)
 
 
 class IterInternalClient(internal_client.InternalClient):
@@ -129,10 +128,10 @@ class IterInternalClient(internal_client.InternalClient):
 
     def _iter_items(
             self, path, marker='', end_marker='', acceptable_statuses=None):
-        self.test.assertEquals(self.path, path)
-        self.test.assertEquals(self.marker, marker)
-        self.test.assertEquals(self.end_marker, end_marker)
-        self.test.assertEquals(self.acceptable_statuses, acceptable_statuses)
+        self.test.assertEqual(self.path, path)
+        self.test.assertEqual(self.marker, marker)
+        self.test.assertEqual(self.end_marker, end_marker)
+        self.test.assertEqual(self.acceptable_statuses, acceptable_statuses)
         for item in self.items:
             yield item
 
@@ -145,7 +144,7 @@ class TestCompressingfileReader(unittest.TestCase):
                 self.args = args
 
             def method(self, *args):
-                self.test.assertEquals(self.args, args)
+                self.test.assertEqual(self.args, args)
                 return self
 
         try:
@@ -158,12 +157,12 @@ class TestCompressingfileReader(unittest.TestCase):
             f = StringIO('')
 
             fobj = internal_client.CompressingFileReader(f)
-            self.assertEquals(f, fobj._f)
-            self.assertEquals(compressobj, fobj._compressor)
-            self.assertEquals(False, fobj.done)
-            self.assertEquals(True, fobj.first)
-            self.assertEquals(0, fobj.crc32)
-            self.assertEquals(0, fobj.total_size)
+            self.assertEqual(f, fobj._f)
+            self.assertEqual(compressobj, fobj._compressor)
+            self.assertEqual(False, fobj.done)
+            self.assertEqual(True, fobj.first)
+            self.assertEqual(0, fobj.crc32)
+            self.assertEqual(0, fobj.total_size)
         finally:
             internal_client.compressobj = old_compressobj
 
@@ -177,7 +176,7 @@ class TestCompressingfileReader(unittest.TestCase):
         for chunk in fobj.read():
             data += d.decompress(chunk)
 
-        self.assertEquals(exp_data, data)
+        self.assertEqual(exp_data, data)
 
     def test_seek(self):
         exp_data = 'abcdefghijklmnopqrstuvwxyz'
@@ -194,7 +193,7 @@ class TestCompressingfileReader(unittest.TestCase):
         d = zlib.decompressobj(16 + zlib.MAX_WBITS)
         for chunk in fobj.read():
             data += d.decompress(chunk)
-        self.assertEquals(exp_data, data)
+        self.assertEqual(exp_data, data)
 
     def test_seek_not_implemented_exception(self):
         fobj = internal_client.CompressingFileReader(
@@ -249,7 +248,7 @@ class TestInternalClient(unittest.TestCase):
                              object_ring)
             self.assertEqual(object_ring.serialized_path,
                              object_ring_path)
-            self.assertEquals(client.auto_create_account_prefix, '-')
+            self.assertEqual(client.auto_create_account_prefix, '-')
 
     def test_init(self):
         class App(object):
@@ -260,7 +259,7 @@ class TestInternalClient(unittest.TestCase):
 
             def load(self, uri, allow_modify_pipeline=True):
                 self.load_called += 1
-                self.test.assertEquals(conf_path, uri)
+                self.test.assertEqual(conf_path, uri)
                 self.test.assertFalse(allow_modify_pipeline)
                 return self
 
@@ -278,10 +277,10 @@ class TestInternalClient(unittest.TestCase):
         finally:
             internal_client.loadapp = old_loadapp
 
-        self.assertEquals(1, app.load_called)
-        self.assertEquals(app, client.app)
-        self.assertEquals(user_agent, client.user_agent)
-        self.assertEquals(request_tries, client.request_tries)
+        self.assertEqual(1, app.load_called)
+        self.assertEqual(app, client.app)
+        self.assertEqual(user_agent, client.user_agent)
+        self.assertEqual(request_tries, client.request_tries)
 
     def test_make_request_sets_user_agent(self):
         class InternalClient(internal_client.InternalClient):
@@ -292,7 +291,7 @@ class TestInternalClient(unittest.TestCase):
                 self.request_tries = 1
 
             def fake_app(self, env, start_response):
-                self.test.assertEquals(self.user_agent, env['HTTP_USER_AGENT'])
+                self.test.assertEqual(self.user_agent, env['HTTP_USER_AGENT'])
                 start_response('200 Ok', [('Content-Length', '0')])
                 return []
 
@@ -320,7 +319,7 @@ class TestInternalClient(unittest.TestCase):
 
             def sleep(self, seconds):
                 self.sleep_called += 1
-                self.test.assertEquals(2 ** (self.sleep_called), seconds)
+                self.test.assertEqual(2 ** (self.sleep_called), seconds)
 
         client = InternalClient(self)
 
@@ -332,8 +331,8 @@ class TestInternalClient(unittest.TestCase):
         finally:
             internal_client.sleep = old_sleep
 
-        self.assertEquals(3, client.sleep_called)
-        self.assertEquals(4, client.tries)
+        self.assertEqual(3, client.sleep_called)
+        self.assertEqual(4, client.tries)
 
     def test_base_request_timeout(self):
         # verify that base_request passes timeout arg on to urlopen
@@ -351,7 +350,7 @@ class TestInternalClient(unittest.TestCase):
                 _, resp_body = sc.base_request('GET', timeout=timeout)
                 mock_urlopen.assert_called_once_with(mock.ANY, timeout=timeout)
                 # sanity check
-                self.assertEquals(body, resp_body)
+                self.assertEqual(body, resp_body)
 
     def test_make_request_method_path_headers(self):
         class InternalClient(internal_client.InternalClient):
@@ -370,12 +369,12 @@ class TestInternalClient(unittest.TestCase):
 
         for method in 'GET PUT HEAD'.split():
             client.make_request(method, '/', {}, (200,))
-            self.assertEquals(client.env['REQUEST_METHOD'], method)
+            self.assertEqual(client.env['REQUEST_METHOD'], method)
 
         for path in '/one /two/three'.split():
             client.make_request('GET', path, {'X-Test': path}, (200,))
-            self.assertEquals(client.env['PATH_INFO'], path)
-            self.assertEquals(client.env['HTTP_X_TEST'], path)
+            self.assertEqual(client.env['PATH_INFO'], path)
+            self.assertEqual(client.env['HTTP_X_TEST'], path)
 
     def test_make_request_codes(self):
         class InternalClient(internal_client.InternalClient):
@@ -403,12 +402,12 @@ class TestInternalClient(unittest.TestCase):
                 client.make_request('GET', '/', {}, (400,))
             except Exception as err:
                 pass
-            self.assertEquals(200, err.resp.status_int)
+            self.assertEqual(200, err.resp.status_int)
             try:
                 client.make_request('GET', '/', {}, (201,))
             except Exception as err:
                 pass
-            self.assertEquals(200, err.resp.status_int)
+            self.assertEqual(200, err.resp.status_int)
             try:
                 client.make_request('GET', '/', {}, (111,))
             except Exception as err:
@@ -426,8 +425,8 @@ class TestInternalClient(unittest.TestCase):
 
             def seek(self, offset, whence=0):
                 self.seek_called += 1
-                self.test.assertEquals(0, offset)
-                self.test.assertEquals(0, whence)
+                self.test.assertEqual(0, offset)
+                self.test.assertEqual(0, whence)
 
         class InternalClient(internal_client.InternalClient):
             def __init__(self):
@@ -449,11 +448,11 @@ class TestInternalClient(unittest.TestCase):
                 client.make_request('PUT', '/', {}, (2,), fobj)
             except Exception as err:
                 pass
-            self.assertEquals(404, err.resp.status_int)
+            self.assertEqual(404, err.resp.status_int)
         finally:
             internal_client.sleep = old_sleep
 
-        self.assertEquals(client.request_tries, fobj.seek_called)
+        self.assertEqual(client.request_tries, fobj.seek_called)
 
     def test_make_request_request_exception(self):
         class InternalClient(internal_client.InternalClient):
@@ -491,10 +490,10 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('HEAD', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals((2,), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('HEAD', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual((2,), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
                 return Response(self.resp_headers)
 
         path = 'some_path'
@@ -514,8 +513,8 @@ class TestInternalClient(unittest.TestCase):
 
         client = InternalClient(self, path, resp_headers)
         metadata = client._get_metadata(path, metadata_prefix)
-        self.assertEquals(exp_metadata, metadata)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(exp_metadata, metadata)
+        self.assertEqual(1, client.make_request_called)
 
     def test_get_metadata_invalid_status(self):
         class FakeApp(object):
@@ -542,7 +541,7 @@ class TestInternalClient(unittest.TestCase):
         path = make_path(account, container, obj)
 
         c = InternalClient()
-        self.assertEquals(path, c.make_path(account, container, obj))
+        self.assertEqual(path, c.make_path(account, container, obj))
 
     def test_make_path_exception(self):
         c = InternalClient()
@@ -572,7 +571,7 @@ class TestInternalClient(unittest.TestCase):
         client = InternalClient(self, responses)
         for item in client._iter_items('/'):
             items.append(item)
-        self.assertEquals(exp_items, items)
+        self.assertEqual(exp_items, items)
 
         exp_items = []
         responses = []
@@ -588,7 +587,7 @@ class TestInternalClient(unittest.TestCase):
         client = InternalClient(self, responses)
         for item in client._iter_items('/'):
             items.append(item)
-        self.assertEquals(exp_items, items)
+        self.assertEqual(exp_items, items)
 
     def test_iter_items_with_markers(self):
         class Response(object):
@@ -606,7 +605,7 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 exp_path = self.paths.pop(0)
-                self.test.assertEquals(exp_path, path)
+                self.test.assertEqual(exp_path, path)
                 return self.responses.pop(0)
 
         paths = [
@@ -626,7 +625,60 @@ class TestInternalClient(unittest.TestCase):
         for item in client._iter_items('/', marker='start', end_marker='end'):
             items.append(item['name'].encode('utf8'))
 
-        self.assertEquals('one\xc3\xa9 two'.split(), items)
+        self.assertEqual('one\xc3\xa9 two'.split(), items)
+
+    def test_iter_item_read_response_if_status_is_acceptable(self):
+        class Response(object):
+            def __init__(self, status_int, body, app_iter):
+                self.status_int = status_int
+                self.body = body
+                self.app_iter = app_iter
+
+        class InternalClient(internal_client.InternalClient):
+            def __init__(self, test, responses):
+                self.test = test
+                self.responses = responses
+
+            def make_request(
+                self, method, path, headers, acceptable_statuses,
+                    body_file=None):
+                resp = self.responses.pop(0)
+                if resp.status_int in acceptable_statuses or \
+                        resp.status_int // 100 in acceptable_statuses:
+                    return resp
+                if resp:
+                    raise internal_client.UnexpectedResponse(
+                        'Unexpected response: %s' % resp.status_int, resp)
+
+        num_list = []
+
+        def generate_resp_body():
+            for i in range(1, 5):
+                yield str(i)
+                num_list.append(i)
+
+        exp_items = []
+        responses = [Response(204, json.dumps([]), generate_resp_body())]
+        items = []
+        client = InternalClient(self, responses)
+        for item in client._iter_items('/'):
+            items.append(item)
+        self.assertEqual(exp_items, items)
+        self.assertEqual(len(num_list), 0)
+
+        responses = [Response(300, json.dumps([]), generate_resp_body())]
+        client = InternalClient(self, responses)
+        self.assertRaises(internal_client.UnexpectedResponse,
+                          next, client._iter_items('/'))
+
+        exp_items = []
+        responses = [Response(404, json.dumps([]), generate_resp_body())]
+        items = []
+        client = InternalClient(self, responses)
+        for item in client._iter_items('/'):
+            items.append(item)
+        self.assertEqual(exp_items, items)
+        self.assertEqual(len(num_list), 4)
 
     def test_set_metadata(self):
         class InternalClient(internal_client.InternalClient):
@@ -640,11 +692,11 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('POST', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals(self.exp_headers, headers)
-                self.test.assertEquals((2,), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('POST', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual(self.exp_headers, headers)
+                self.test.assertEqual((2,), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
 
         path = 'some_path'
         metadata_prefix = 'some_key-'
@@ -661,7 +713,7 @@ class TestInternalClient(unittest.TestCase):
 
         client = InternalClient(self, path, exp_headers)
         client._set_metadata(path, metadata, metadata_prefix)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
     def test_iter_containers(self):
         account, container, obj = path_parts()
@@ -677,7 +729,7 @@ class TestInternalClient(unittest.TestCase):
                 account, marker, end_marker,
                 acceptable_statuses=acceptable_statuses):
             ret_items.append(container)
-        self.assertEquals(items, ret_items)
+        self.assertEqual(items, ret_items)
 
     def test_get_account_info(self):
         class Response(object):
@@ -697,11 +749,11 @@ class TestInternalClient(unittest.TestCase):
             def make_request(
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
-                self.test.assertEquals('HEAD', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals({}, headers)
-                self.test.assertEquals((2, 404), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('HEAD', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual({}, headers)
+                self.test.assertEqual((2, 404), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
                 return self.resp
 
         account, container, obj = path_parts()
@@ -709,7 +761,7 @@ class TestInternalClient(unittest.TestCase):
         containers, objects = 10, 100
         client = InternalClient(self, path, Response(containers, objects))
         info = client.get_account_info(account)
-        self.assertEquals((containers, objects), info)
+        self.assertEqual((containers, objects), info)
 
     def test_get_account_info_404(self):
         class Response(object):
@@ -732,7 +784,7 @@ class TestInternalClient(unittest.TestCase):
 
         client = InternalClient()
         info = client.get_account_info('some_account')
-        self.assertEquals((0, 0), info)
+        self.assertEqual((0, 0), info)
 
     def test_get_account_metadata(self):
         account, container, obj = path_parts()
@@ -743,8 +795,8 @@ class TestInternalClient(unittest.TestCase):
             self, path, metadata_prefix, acceptable_statuses)
         metadata = client.get_account_metadata(
             account, metadata_prefix, acceptable_statuses)
-        self.assertEquals(client.metadata, metadata)
-        self.assertEquals(1, client.get_metadata_called)
+        self.assertEqual(client.metadata, metadata)
+        self.assertEqual(1, client.get_metadata_called)
 
     def test_get_metadadata_with_acceptable_status(self):
         account, container, obj = path_parts()
@@ -776,7 +828,7 @@ class TestInternalClient(unittest.TestCase):
             self, path, metadata, metadata_prefix, acceptable_statuses)
         client.set_account_metadata(
             account, metadata, metadata_prefix, acceptable_statuses)
-        self.assertEquals(1, client.set_metadata_called)
+        self.assertEqual(1, client.set_metadata_called)
 
     def test_container_exists(self):
         class Response(object):
@@ -794,23 +846,23 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('HEAD', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals({}, headers)
-                self.test.assertEquals((2, 404), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('HEAD', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual({}, headers)
+                self.test.assertEqual((2, 404), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
                 return self.resp
 
         account, container, obj = path_parts()
         path = make_path(account, container)
 
         client = InternalClient(self, path, Response(200))
-        self.assertEquals(True, client.container_exists(account, container))
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(True, client.container_exists(account, container))
+        self.assertEqual(1, client.make_request_called)
 
         client = InternalClient(self, path, Response(404))
-        self.assertEquals(False, client.container_exists(account, container))
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(False, client.container_exists(account, container))
+        self.assertEqual(1, client.make_request_called)
 
     def test_create_container(self):
         class InternalClient(internal_client.InternalClient):
@@ -824,18 +876,18 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('PUT', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals(self.headers, headers)
-                self.test.assertEquals((2,), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('PUT', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual(self.headers, headers)
+                self.test.assertEqual((2,), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
 
         account, container, obj = path_parts()
         path = make_path(account, container)
         headers = 'some_headers'
         client = InternalClient(self, path, headers)
         client.create_container(account, container, headers)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
     def test_delete_container(self):
         class InternalClient(internal_client.InternalClient):
@@ -848,17 +900,17 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('DELETE', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals({}, headers)
-                self.test.assertEquals((2, 404), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('DELETE', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual({}, headers)
+                self.test.assertEqual((2, 404), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
 
         account, container, obj = path_parts()
         path = make_path(account, container)
         client = InternalClient(self, path)
         client.delete_container(account, container)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
     def test_get_container_metadata(self):
         account, container, obj = path_parts()
@@ -869,8 +921,8 @@ class TestInternalClient(unittest.TestCase):
             self, path, metadata_prefix, acceptable_statuses)
         metadata = client.get_container_metadata(
             account, container, metadata_prefix, acceptable_statuses)
-        self.assertEquals(client.metadata, metadata)
-        self.assertEquals(1, client.get_metadata_called)
+        self.assertEqual(client.metadata, metadata)
+        self.assertEqual(1, client.get_metadata_called)
 
     def test_iter_objects(self):
         account, container, obj = path_parts()
@@ -885,7 +937,7 @@ class TestInternalClient(unittest.TestCase):
         for obj in client.iter_objects(
                 account, container, marker, end_marker, acceptable_statuses):
             ret_items.append(obj)
-        self.assertEquals(items, ret_items)
+        self.assertEqual(items, ret_items)
 
     def test_set_container_metadata(self):
         account, container, obj = path_parts()
@@ -897,7 +949,7 @@ class TestInternalClient(unittest.TestCase):
             self, path, metadata, metadata_prefix, acceptable_statuses)
         client.set_container_metadata(
             account, container, metadata, metadata_prefix, acceptable_statuses)
-        self.assertEquals(1, client.set_metadata_called)
+        self.assertEqual(1, client.set_metadata_called)
 
     def test_delete_object(self):
         class InternalClient(internal_client.InternalClient):
@@ -910,18 +962,18 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals('DELETE', method)
-                self.test.assertEquals(self.path, path)
-                self.test.assertEquals({}, headers)
-                self.test.assertEquals((2, 404), acceptable_statuses)
-                self.test.assertEquals(None, body_file)
+                self.test.assertEqual('DELETE', method)
+                self.test.assertEqual(self.path, path)
+                self.test.assertEqual({}, headers)
+                self.test.assertEqual((2, 404), acceptable_statuses)
+                self.test.assertEqual(None, body_file)
 
         account, container, obj = path_parts()
         path = make_path(account, container, obj)
 
         client = InternalClient(self, path)
         client.delete_object(account, container, obj)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
     def test_get_object_metadata(self):
         account, container, obj = path_parts()
@@ -933,8 +985,8 @@ class TestInternalClient(unittest.TestCase):
         metadata = client.get_object_metadata(
             account, container, obj, metadata_prefix,
             acceptable_statuses)
-        self.assertEquals(client.metadata, metadata)
-        self.assertEquals(1, client.get_metadata_called)
+        self.assertEqual(client.metadata, metadata)
+        self.assertEqual(1, client.get_metadata_called)
 
     def test_get_metadata_extra_headers(self):
         class InternalClient(internal_client.InternalClient):
@@ -994,7 +1046,7 @@ class TestInternalClient(unittest.TestCase):
         ret_lines = []
         for line in client.iter_object_lines('account', 'container', 'object'):
             ret_lines.append(line)
-        self.assertEquals(lines, ret_lines)
+        self.assertEqual(lines, ret_lines)
 
     def test_iter_object_lines_compressed_object(self):
         class InternalClient(internal_client.InternalClient):
@@ -1015,7 +1067,7 @@ class TestInternalClient(unittest.TestCase):
         for line in client.iter_object_lines(
                 'account', 'container', 'object.gz'):
             ret_lines.append(line)
-        self.assertEquals(lines, ret_lines)
+        self.assertEqual(lines, ret_lines)
 
     def test_iter_object_lines_404(self):
         class InternalClient(internal_client.InternalClient):
@@ -1034,7 +1086,7 @@ class TestInternalClient(unittest.TestCase):
                 'some_account', 'some_container', 'some_object',
                 acceptable_statuses=(2, 404)):
             lines.append(line)
-        self.assertEquals([], lines)
+        self.assertEqual([], lines)
 
     def test_set_object_metadata(self):
         account, container, obj = path_parts()
@@ -1047,7 +1099,7 @@ class TestInternalClient(unittest.TestCase):
         client.set_object_metadata(
             account, container, obj, metadata, metadata_prefix,
             acceptable_statuses)
-        self.assertEquals(1, client.set_metadata_called)
+        self.assertEqual(1, client.set_metadata_called)
 
     def test_upload_object(self):
         class InternalClient(internal_client.InternalClient):
@@ -1062,11 +1114,11 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals(self.path, path)
+                self.test.assertEqual(self.path, path)
                 exp_headers = dict(self.headers)
                 exp_headers['Transfer-Encoding'] = 'chunked'
-                self.test.assertEquals(exp_headers, headers)
-                self.test.assertEquals(self.fobj, fobj)
+                self.test.assertEqual(exp_headers, headers)
+                self.test.assertEqual(self.fobj, fobj)
 
         fobj = 'some_fobj'
         account, container, obj = path_parts()
@@ -1075,7 +1127,7 @@ class TestInternalClient(unittest.TestCase):
 
         client = InternalClient(self, path, headers, fobj)
         client.upload_object(fobj, account, container, obj, headers)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
     def test_upload_object_not_chunked(self):
         class InternalClient(internal_client.InternalClient):
@@ -1090,10 +1142,10 @@ class TestInternalClient(unittest.TestCase):
                     self, method, path, headers, acceptable_statuses,
                     body_file=None):
                 self.make_request_called += 1
-                self.test.assertEquals(self.path, path)
+                self.test.assertEqual(self.path, path)
                 exp_headers = dict(self.headers)
-                self.test.assertEquals(exp_headers, headers)
-                self.test.assertEquals(self.fobj, fobj)
+                self.test.assertEqual(exp_headers, headers)
+                self.test.assertEqual(self.fobj, fobj)
 
         fobj = 'some_fobj'
         account, container, obj = path_parts()
@@ -1102,7 +1154,7 @@ class TestInternalClient(unittest.TestCase):
 
         client = InternalClient(self, path, headers, fobj)
         client.upload_object(fobj, account, container, obj, headers)
-        self.assertEquals(1, client.make_request_called)
+        self.assertEqual(1, client.make_request_called)
 
 
 class TestGetAuth(unittest.TestCase):
@@ -1276,7 +1328,7 @@ class TestSimpleClient(unittest.TestCase):
             mock_urlopen.side_effect = urllib2.HTTPError(*[None] * 5)
             with mock.patch('swift.common.internal_client.sleep') \
                     as mock_sleep:
-                self.assertRaises(urllib2.HTTPError,
+                self.assertRaises(exceptions.ClientException,
                                   c.retry_request, request_method, retries=1)
             self.assertEqual(mock_sleep.call_count, 1)
             self.assertEqual(mock_urlopen.call_count, 2)
@@ -1294,7 +1346,7 @@ class TestSimpleClient(unittest.TestCase):
             mock_urlopen.side_effect = urllib2.HTTPError(*[None] * 5)
             with mock.patch('swift.common.internal_client.sleep') \
                     as mock_sleep:
-                self.assertRaises(urllib2.HTTPError,
+                self.assertRaises(exceptions.ClientException,
                                   c.retry_request, request_method,
                                   container='con', retries=1)
             self.assertEqual(mock_sleep.call_count, 1)
@@ -1313,7 +1365,7 @@ class TestSimpleClient(unittest.TestCase):
             mock_urlopen.side_effect = urllib2.HTTPError(*[None] * 5)
             with mock.patch('swift.common.internal_client.sleep') \
                     as mock_sleep:
-                self.assertRaises(urllib2.HTTPError,
+                self.assertRaises(exceptions.ClientException,
                                   c.retry_request, request_method,
                                   container='con', name='obj', retries=1)
             self.assertEqual(mock_sleep.call_count, 1)
