@@ -22,13 +22,12 @@ import socket
 import unittest
 import os
 from textwrap import dedent
-from contextlib import nested
 from collections import defaultdict
-from urllib import quote
 
 from eventlet import listen
 from six import BytesIO
 from six import StringIO
+from six.moves.urllib.parse import quote
 
 import mock
 
@@ -73,41 +72,41 @@ class TestWSGI(unittest.TestCase):
 
     def test_monkey_patch_mimetools(self):
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).type, 'text/plain')
+        self.assertEqual(mimetools.Message(sio).type, 'text/plain')
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).plisttext, '')
+        self.assertEqual(mimetools.Message(sio).plisttext, '')
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).maintype, 'text')
+        self.assertEqual(mimetools.Message(sio).maintype, 'text')
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).subtype, 'plain')
+        self.assertEqual(mimetools.Message(sio).subtype, 'plain')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).type, 'text/html')
+        self.assertEqual(mimetools.Message(sio).type, 'text/html')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).plisttext,
-                          '; charset=ISO-8859-4')
+        self.assertEqual(mimetools.Message(sio).plisttext,
+                         '; charset=ISO-8859-4')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).maintype, 'text')
+        self.assertEqual(mimetools.Message(sio).maintype, 'text')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).subtype, 'html')
+        self.assertEqual(mimetools.Message(sio).subtype, 'html')
 
         wsgi.monkey_patch_mimetools()
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).type, None)
+        self.assertEqual(mimetools.Message(sio).type, None)
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).plisttext, '')
+        self.assertEqual(mimetools.Message(sio).plisttext, '')
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).maintype, None)
+        self.assertEqual(mimetools.Message(sio).maintype, None)
         sio = StringIO('blah')
-        self.assertEquals(mimetools.Message(sio).subtype, None)
+        self.assertEqual(mimetools.Message(sio).subtype, None)
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).type, 'text/html')
+        self.assertEqual(mimetools.Message(sio).type, 'text/html')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).plisttext,
-                          '; charset=ISO-8859-4')
+        self.assertEqual(mimetools.Message(sio).plisttext,
+                         '; charset=ISO-8859-4')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).maintype, 'text')
+        self.assertEqual(mimetools.Message(sio).maintype, 'text')
         sio = StringIO('Content-Type: text/html; charset=ISO-8859-4')
-        self.assertEquals(mimetools.Message(sio).subtype, 'html')
+        self.assertEqual(mimetools.Message(sio).subtype, 'html')
 
     def test_init_request_processor(self):
         config = """
@@ -150,7 +149,7 @@ class TestWSGI(unittest.TestCase):
         expected = swift.proxy.server.Application
         self.assertTrue(isinstance(app, expected))
         # config settings applied to app instance
-        self.assertEquals(0.2, app.conn_timeout)
+        self.assertEqual(0.2, app.conn_timeout)
         # appconfig returns values from 'proxy-server' section
         expected = {
             '__file__': conf_file,
@@ -158,10 +157,10 @@ class TestWSGI(unittest.TestCase):
             'conn_timeout': '0.2',
             'swift_dir': t,
         }
-        self.assertEquals(expected, conf)
+        self.assertEqual(expected, conf)
         # logger works
         logger.info('testing')
-        self.assertEquals('proxy-server', log_name)
+        self.assertEqual('proxy-server', log_name)
 
     @with_tempdir
     def test_loadapp_from_file(self, tempdir):
@@ -215,7 +214,7 @@ class TestWSGI(unittest.TestCase):
         self.assertTrue(isinstance(app, expected))
         self.assertTrue(isinstance(app.app, swift.proxy.server.Application))
         # config settings applied to app instance
-        self.assertEquals(0.2, app.app.conn_timeout)
+        self.assertEqual(0.2, app.app.conn_timeout)
         # appconfig returns values from 'proxy-server' section
         expected = {
             '__file__': conf_dir,
@@ -223,10 +222,10 @@ class TestWSGI(unittest.TestCase):
             'conn_timeout': '0.2',
             'swift_dir': conf_root,
         }
-        self.assertEquals(expected, conf)
+        self.assertEqual(expected, conf)
         # logger works
         logger.info('testing')
-        self.assertEquals('proxy-server', log_name)
+        self.assertEqual('proxy-server', log_name)
 
     def test_get_socket_bad_values(self):
         # first try with no port set
@@ -287,14 +286,14 @@ class TestWSGI(unittest.TestCase):
             if hasattr(socket, 'TCP_KEEPIDLE'):
                 expected_socket_opts[socket.IPPROTO_TCP][
                     socket.TCP_KEEPIDLE] = 600
-            self.assertEquals(sock.opts, expected_socket_opts)
+            self.assertEqual(sock.opts, expected_socket_opts)
             # test ssl
             sock = wsgi.get_socket(ssl_conf)
             expected_kwargs = {
                 'certfile': '',
                 'keyfile': '',
             }
-            self.assertEquals(wsgi.ssl.wrap_socket_called, [expected_kwargs])
+            self.assertEqual(wsgi.ssl.wrap_socket_called, [expected_kwargs])
         finally:
             wsgi.listen = old_listen
             wsgi.ssl = old_ssl
@@ -375,22 +374,23 @@ class TestWSGI(unittest.TestCase):
                             logger = logging.getLogger('test')
                             sock = listen(('localhost', 0))
                             wsgi.run_server(conf, logger, sock)
-        self.assertEquals('HTTP/1.0',
-                          _wsgi.HttpProtocol.default_request_version)
-        self.assertEquals(30, _wsgi.WRITE_TIMEOUT)
+        self.assertEqual('HTTP/1.0',
+                         _wsgi.HttpProtocol.default_request_version)
+        self.assertEqual(30, _wsgi.WRITE_TIMEOUT)
         _eventlet.hubs.use_hub.assert_called_with(utils.get_hub())
         _eventlet.patcher.monkey_patch.assert_called_with(all=False,
-                                                          socket=True)
+                                                          socket=True,
+                                                          thread=True)
         _eventlet.debug.hub_exceptions.assert_called_with(False)
         self.assertTrue(_wsgi.server.called)
         args, kwargs = _wsgi.server.call_args
         server_sock, server_app, server_logger = args
-        self.assertEquals(sock, server_sock)
+        self.assertEqual(sock, server_sock)
         self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
-        self.assertEquals(20, server_app.client_timeout)
+        self.assertEqual(20, server_app.client_timeout)
         self.assertTrue(isinstance(server_logger, wsgi.NullLogger))
         self.assertTrue('custom_pool' in kwargs)
-        self.assertEquals(1000, kwargs['custom_pool'].size)
+        self.assertEqual(1000, kwargs['custom_pool'].size)
 
     def test_run_server_with_latest_eventlet(self):
         config = """
@@ -413,13 +413,12 @@ class TestWSGI(unittest.TestCase):
             with open(conf_file, 'w') as f:
                 f.write(contents.replace('TEMPDIR', t))
             _fake_rings(t)
-            with nested(
-                mock.patch('swift.proxy.server.Application.'
-                           'modify_wsgi_pipeline'),
-                mock.patch('swift.common.wsgi.wsgi'),
-                mock.patch('swift.common.wsgi.eventlet'),
-                mock.patch('swift.common.wsgi.inspect',
-                           getargspec=argspec_stub)) as (_, _wsgi, _, _):
+            with mock.patch('swift.proxy.server.Application.'
+                            'modify_wsgi_pipeline'), \
+                    mock.patch('swift.common.wsgi.wsgi') as _wsgi, \
+                    mock.patch('swift.common.wsgi.eventlet'), \
+                    mock.patch('swift.common.wsgi.inspect',
+                               getargspec=argspec_stub):
                 conf = wsgi.appconfig(conf_file)
                 logger = logging.getLogger('test')
                 sock = listen(('localhost', 0))
@@ -427,7 +426,7 @@ class TestWSGI(unittest.TestCase):
 
         self.assertTrue(_wsgi.server.called)
         args, kwargs = _wsgi.server.call_args
-        self.assertEquals(kwargs.get('capitalize_response_headers'), False)
+        self.assertEqual(kwargs.get('capitalize_response_headers'), False)
 
     def test_run_server_conf_dir(self):
         config_dir = {
@@ -463,17 +462,18 @@ class TestWSGI(unittest.TestCase):
                                 wsgi.run_server(conf, logger, sock)
                                 self.assertTrue(os.environ['TZ'] is not '')
 
-        self.assertEquals('HTTP/1.0',
-                          _wsgi.HttpProtocol.default_request_version)
-        self.assertEquals(30, _wsgi.WRITE_TIMEOUT)
+        self.assertEqual('HTTP/1.0',
+                         _wsgi.HttpProtocol.default_request_version)
+        self.assertEqual(30, _wsgi.WRITE_TIMEOUT)
         _eventlet.hubs.use_hub.assert_called_with(utils.get_hub())
         _eventlet.patcher.monkey_patch.assert_called_with(all=False,
-                                                          socket=True)
+                                                          socket=True,
+                                                          thread=True)
         _eventlet.debug.hub_exceptions.assert_called_with(False)
         self.assertTrue(_wsgi.server.called)
         args, kwargs = _wsgi.server.call_args
         server_sock, server_app, server_logger = args
-        self.assertEquals(sock, server_sock)
+        self.assertEqual(sock, server_sock)
         self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
         self.assertTrue(isinstance(server_logger, wsgi.NullLogger))
         self.assertTrue('custom_pool' in kwargs)
@@ -514,22 +514,23 @@ class TestWSGI(unittest.TestCase):
                         logger = logging.getLogger('test')
                         sock = listen(('localhost', 0))
                         wsgi.run_server(conf, logger, sock)
-        self.assertEquals('HTTP/1.0',
-                          _wsgi.HttpProtocol.default_request_version)
-        self.assertEquals(30, _wsgi.WRITE_TIMEOUT)
+        self.assertEqual('HTTP/1.0',
+                         _wsgi.HttpProtocol.default_request_version)
+        self.assertEqual(30, _wsgi.WRITE_TIMEOUT)
         _eventlet.hubs.use_hub.assert_called_with(utils.get_hub())
         _eventlet.patcher.monkey_patch.assert_called_with(all=False,
-                                                          socket=True)
+                                                          socket=True,
+                                                          thread=True)
         _eventlet.debug.hub_exceptions.assert_called_with(True)
         self.assertTrue(mock_server.called)
         args, kwargs = mock_server.call_args
         server_sock, server_app, server_logger = args
-        self.assertEquals(sock, server_sock)
+        self.assertEqual(sock, server_sock)
         self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
-        self.assertEquals(20, server_app.client_timeout)
+        self.assertEqual(20, server_app.client_timeout)
         self.assertEqual(server_logger, None)
         self.assertTrue('custom_pool' in kwargs)
-        self.assertEquals(1000, kwargs['custom_pool'].size)
+        self.assertEqual(1000, kwargs['custom_pool'].size)
 
     def test_appconfig_dir_ignores_hidden_files(self):
         config_dir = {
@@ -554,26 +555,26 @@ class TestWSGI(unittest.TestCase):
             'here': os.path.join(path, 'server.conf.d'),
             'port': '8080',
         }
-        self.assertEquals(conf, expected)
+        self.assertEqual(conf, expected)
 
     def test_pre_auth_wsgi_input(self):
         oldenv = {}
         newenv = wsgi.make_pre_authed_env(oldenv)
         self.assertTrue('wsgi.input' in newenv)
-        self.assertEquals(newenv['wsgi.input'].read(), '')
+        self.assertEqual(newenv['wsgi.input'].read(), '')
 
         oldenv = {'wsgi.input': BytesIO(b'original wsgi.input')}
         newenv = wsgi.make_pre_authed_env(oldenv)
         self.assertTrue('wsgi.input' in newenv)
-        self.assertEquals(newenv['wsgi.input'].read(), '')
+        self.assertEqual(newenv['wsgi.input'].read(), '')
 
         oldenv = {'swift.source': 'UT'}
         newenv = wsgi.make_pre_authed_env(oldenv)
-        self.assertEquals(newenv['swift.source'], 'UT')
+        self.assertEqual(newenv['swift.source'], 'UT')
 
         oldenv = {'swift.source': 'UT'}
         newenv = wsgi.make_pre_authed_env(oldenv, swift_source='SA')
-        self.assertEquals(newenv['swift.source'], 'SA')
+        self.assertEqual(newenv['swift.source'], 'SA')
 
     def test_pre_auth_req(self):
         class FakeReq(object):
@@ -583,7 +584,7 @@ class TestWSGI(unittest.TestCase):
                     environ = {}
                 if headers is None:
                     headers = {}
-                self.assertEquals(environ['swift.authorize']('test'), None)
+                self.assertEqual(environ['swift.authorize']('test'), None)
                 self.assertFalse('HTTP_X_TRANS_ID' in environ)
         was_blank = Request.blank
         Request.blank = FakeReq.fake_blank
@@ -597,23 +598,23 @@ class TestWSGI(unittest.TestCase):
         r = wsgi.make_pre_authed_request(
             {'HTTP_X_TRANS_ID': '1234'}, 'PUT', path=quote('/a space'),
             body='tester', headers={})
-        self.assertEquals(r.path, quote('/a space'))
+        self.assertEqual(r.path, quote('/a space'))
 
     def test_pre_auth_req_drops_query(self):
         r = wsgi.make_pre_authed_request(
             {'QUERY_STRING': 'original'}, 'GET', 'path')
-        self.assertEquals(r.query_string, 'original')
+        self.assertEqual(r.query_string, 'original')
         r = wsgi.make_pre_authed_request(
             {'QUERY_STRING': 'original'}, 'GET', 'path?replacement')
-        self.assertEquals(r.query_string, 'replacement')
+        self.assertEqual(r.query_string, 'replacement')
         r = wsgi.make_pre_authed_request(
             {'QUERY_STRING': 'original'}, 'GET', 'path?')
-        self.assertEquals(r.query_string, '')
+        self.assertEqual(r.query_string, '')
 
     def test_pre_auth_req_with_body(self):
         r = wsgi.make_pre_authed_request(
             {'QUERY_STRING': 'original'}, 'GET', 'path', 'the body')
-        self.assertEquals(r.body, 'the body')
+        self.assertEqual(r.body, 'the body')
 
     def test_pre_auth_creates_script_name(self):
         e = wsgi.make_pre_authed_env({})
@@ -621,20 +622,20 @@ class TestWSGI(unittest.TestCase):
 
     def test_pre_auth_copies_script_name(self):
         e = wsgi.make_pre_authed_env({'SCRIPT_NAME': '/script_name'})
-        self.assertEquals(e['SCRIPT_NAME'], '/script_name')
+        self.assertEqual(e['SCRIPT_NAME'], '/script_name')
 
     def test_pre_auth_copies_script_name_unless_path_overridden(self):
         e = wsgi.make_pre_authed_env({'SCRIPT_NAME': '/script_name'},
                                      path='/override')
-        self.assertEquals(e['SCRIPT_NAME'], '')
-        self.assertEquals(e['PATH_INFO'], '/override')
+        self.assertEqual(e['SCRIPT_NAME'], '')
+        self.assertEqual(e['PATH_INFO'], '/override')
 
     def test_pre_auth_req_swift_source(self):
         r = wsgi.make_pre_authed_request(
             {'QUERY_STRING': 'original'}, 'GET', 'path', 'the body',
             swift_source='UT')
-        self.assertEquals(r.body, 'the body')
-        self.assertEquals(r.environ['swift.source'], 'UT')
+        self.assertEqual(r.body, 'the body')
+        self.assertEqual(r.environ['swift.source'], 'UT')
 
     def test_run_server_global_conf_callback(self):
         calls = defaultdict(lambda: 0)
@@ -658,15 +659,15 @@ class TestWSGI(unittest.TestCase):
             self.assertEqual(kwargs['global_conf'],
                              {'log_name': 'log_name', 'test1': 'one'})
 
-        with nested(
-                mock.patch.object(wsgi, '_initrp', _initrp),
-                mock.patch.object(wsgi, 'get_socket'),
-                mock.patch.object(wsgi, 'drop_privileges'),
-                mock.patch.object(wsgi, 'loadapp', _loadapp),
-                mock.patch.object(wsgi, 'capture_stdio'),
-                mock.patch.object(wsgi, 'run_server')):
+        with mock.patch.object(wsgi, '_initrp', _initrp), \
+                mock.patch.object(wsgi, 'get_socket'), \
+                mock.patch.object(wsgi, 'drop_privileges'), \
+                mock.patch.object(wsgi, 'loadapp', _loadapp), \
+                mock.patch.object(wsgi, 'capture_stdio'), \
+                mock.patch.object(wsgi, 'run_server'):
             wsgi.run_wsgi('conf_file', 'app_section',
                           global_conf_callback=_global_conf_callback)
+
         self.assertEqual(calls['_global_conf_callback'], 1)
         self.assertEqual(calls['_loadapp'], 1)
 
@@ -683,13 +684,12 @@ class TestWSGI(unittest.TestCase):
         def _loadapp(uri, name=None, **kwargs):
             calls['_loadapp'] += 1
 
-        with nested(
-                mock.patch.object(wsgi, '_initrp', _initrp),
-                mock.patch.object(wsgi, 'get_socket'),
-                mock.patch.object(wsgi, 'drop_privileges'),
-                mock.patch.object(wsgi, 'loadapp', _loadapp),
-                mock.patch.object(wsgi, 'capture_stdio'),
-                mock.patch.object(wsgi, 'run_server')):
+        with mock.patch.object(wsgi, '_initrp', _initrp), \
+                mock.patch.object(wsgi, 'get_socket'), \
+                mock.patch.object(wsgi, 'drop_privileges'), \
+                mock.patch.object(wsgi, 'loadapp', _loadapp), \
+                mock.patch.object(wsgi, 'capture_stdio'), \
+                mock.patch.object(wsgi, 'run_server'):
             rc = wsgi.run_wsgi('conf_file', 'app_section')
         self.assertEqual(calls['_initrp'], 1)
         self.assertEqual(calls['_loadapp'], 1)
@@ -764,13 +764,12 @@ class TestWSGI(unittest.TestCase):
         def _loadapp(uri, name=None, **kwargs):
             calls['_loadapp'] += 1
 
-        with nested(
-                mock.patch.object(wsgi, '_initrp', _initrp),
-                mock.patch.object(wsgi, 'get_socket'),
-                mock.patch.object(wsgi, 'drop_privileges'),
-                mock.patch.object(wsgi, 'loadapp', _loadapp),
-                mock.patch.object(wsgi, 'capture_stdio'),
-                mock.patch.object(wsgi, 'run_server')):
+        with mock.patch.object(wsgi, '_initrp', _initrp), \
+                mock.patch.object(wsgi, 'get_socket'), \
+                mock.patch.object(wsgi, 'drop_privileges'), \
+                mock.patch.object(wsgi, 'loadapp', _loadapp), \
+                mock.patch.object(wsgi, 'capture_stdio'), \
+                mock.patch.object(wsgi, 'run_server'):
             rc = wsgi.run_wsgi('conf_file', 'app_section')
         self.assertEqual(calls['_initrp'], 1)
         self.assertEqual(calls['_loadapp'], 0)
@@ -779,52 +778,59 @@ class TestWSGI(unittest.TestCase):
     def test_pre_auth_req_with_empty_env_no_path(self):
         r = wsgi.make_pre_authed_request(
             {}, 'GET')
-        self.assertEquals(r.path, quote(''))
+        self.assertEqual(r.path, quote(''))
         self.assertTrue('SCRIPT_NAME' in r.environ)
         self.assertTrue('PATH_INFO' in r.environ)
 
     def test_pre_auth_req_with_env_path(self):
         r = wsgi.make_pre_authed_request(
             {'PATH_INFO': '/unquoted path with %20'}, 'GET')
-        self.assertEquals(r.path, quote('/unquoted path with %20'))
-        self.assertEquals(r.environ['SCRIPT_NAME'], '')
+        self.assertEqual(r.path, quote('/unquoted path with %20'))
+        self.assertEqual(r.environ['SCRIPT_NAME'], '')
 
     def test_pre_auth_req_with_env_script(self):
         r = wsgi.make_pre_authed_request({'SCRIPT_NAME': '/hello'}, 'GET')
-        self.assertEquals(r.path, quote('/hello'))
+        self.assertEqual(r.path, quote('/hello'))
 
     def test_pre_auth_req_with_env_path_and_script(self):
         env = {'PATH_INFO': '/unquoted path with %20',
                'SCRIPT_NAME': '/script'}
         r = wsgi.make_pre_authed_request(env, 'GET')
         expected_path = quote(env['SCRIPT_NAME'] + env['PATH_INFO'])
-        self.assertEquals(r.path, expected_path)
+        self.assertEqual(r.path, expected_path)
         env = {'PATH_INFO': '', 'SCRIPT_NAME': '/script'}
         r = wsgi.make_pre_authed_request(env, 'GET')
-        self.assertEquals(r.path, '/script')
+        self.assertEqual(r.path, '/script')
         env = {'PATH_INFO': '/path', 'SCRIPT_NAME': ''}
         r = wsgi.make_pre_authed_request(env, 'GET')
-        self.assertEquals(r.path, '/path')
+        self.assertEqual(r.path, '/path')
         env = {'PATH_INFO': '', 'SCRIPT_NAME': ''}
         r = wsgi.make_pre_authed_request(env, 'GET')
-        self.assertEquals(r.path, '')
+        self.assertEqual(r.path, '')
 
     def test_pre_auth_req_path_overrides_env(self):
         env = {'PATH_INFO': '/path', 'SCRIPT_NAME': '/script'}
         r = wsgi.make_pre_authed_request(env, 'GET', '/override')
-        self.assertEquals(r.path, '/override')
-        self.assertEquals(r.environ['SCRIPT_NAME'], '')
-        self.assertEquals(r.environ['PATH_INFO'], '/override')
+        self.assertEqual(r.path, '/override')
+        self.assertEqual(r.environ['SCRIPT_NAME'], '')
+        self.assertEqual(r.environ['PATH_INFO'], '/override')
 
     def test_make_env_keep_user_project_id(self):
         oldenv = {'HTTP_X_USER_ID': '1234', 'HTTP_X_PROJECT_ID': '5678'}
         newenv = wsgi.make_env(oldenv)
 
         self.assertTrue('HTTP_X_USER_ID' in newenv)
-        self.assertEquals(newenv['HTTP_X_USER_ID'], '1234')
+        self.assertEqual(newenv['HTTP_X_USER_ID'], '1234')
 
         self.assertTrue('HTTP_X_PROJECT_ID' in newenv)
-        self.assertEquals(newenv['HTTP_X_PROJECT_ID'], '5678')
+        self.assertEqual(newenv['HTTP_X_PROJECT_ID'], '5678')
+
+    def test_make_env_keeps_referer(self):
+        oldenv = {'HTTP_REFERER': 'http://blah.example.com'}
+        newenv = wsgi.make_env(oldenv)
+
+        self.assertTrue('HTTP_REFERER' in newenv)
+        self.assertEqual(newenv['HTTP_REFERER'], 'http://blah.example.com')
 
 
 class TestServersPerPortStrategy(unittest.TestCase):
@@ -1214,12 +1220,12 @@ class TestWSGIContext(unittest.TestCase):
         wc = wsgi.WSGIContext(app)
         r = Request.blank('/')
         it = wc._app_call(r.environ)
-        self.assertEquals(wc._response_status, '200 Ok')
-        self.assertEquals(''.join(it), 'Ok\n')
+        self.assertEqual(wc._response_status, '200 Ok')
+        self.assertEqual(''.join(it), 'Ok\n')
         r = Request.blank('/')
         it = wc._app_call(r.environ)
-        self.assertEquals(wc._response_status, '404 Not Found')
-        self.assertEquals(''.join(it), 'Ok\n')
+        self.assertEqual(wc._response_status, '404 Not Found')
+        self.assertEqual(''.join(it), 'Ok\n')
 
     def test_app_iter_is_closable(self):
 
@@ -1234,7 +1240,7 @@ class TestWSGIContext(unittest.TestCase):
         wc = wsgi.WSGIContext(app)
         r = Request.blank('/')
         iterable = wc._app_call(r.environ)
-        self.assertEquals(wc._response_status, '200 OK')
+        self.assertEqual(wc._response_status, '200 OK')
 
         iterator = iter(iterable)
         self.assertEqual('aaaaa', next(iterator))
